/******************************************************************************
 * Licensed Materials - Property of IBM                                       *
 *                                                                            *
 * PID (5697-TKT)                                                             *
 *                                                                            *
 * (C) Copyright by IBM 2000. All Rights Reserved.                            *
 *****************************************************************************/
package com.BobCo.postit;

import java.awt.event.ActionEvent;
import java.util.ListResourceBundle;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPopupMenu;
import com.tivoli.twg.engine.TWGConObject;
import com.tivoli.twg.engine.TWGConContentInfoObject;
import com.tivoli.twg.engine.TWGContentInfoEventHandler;
import com.tivoli.twg.libs.BadServiceNodeImplException;
import com.tivoli.twg.libs.Command;
import com.tivoli.twg.libs.CommandCompleteListener;
import com.tivoli.twg.libs.LongValueSet;
import com.tivoli.twg.libs.ServiceNode;
import com.tivoli.twg.libs.ServiceNodeException;
import com.tivoli.twg.libs.ServiceNodeClosedException;

/*****************************************************************************
*                         Bob's Software House
*                    A Division of BobCo Enterprises
*****************************************************************************/
/****************************************************************************
* The TWGContentInfoEventHandler class defines the interface that is used
* by the console to notify a task association's view about user interaction.
* This interaction includes popup menu requests, double-click actions,
* menu action requests and drag/drop operations.
*****************************************************************************/
public class BCPostItContentInfoEventHandler extends TWGContentInfoEventHandler
                                             implements CommandCompleteListener
{
    /**
     * Construct the handler object.  Default ctor so that it can be loaded
     * as a bean.
     */
    public BCPostItContentInfoEventHandler()
    {
        super();

        // Obtain access to the samples specific NLS resources
        bcpiResources = (ListResourceBundle)ListResourceBundle.
                        getBundle( "com.BobCo.postit.BCPostItResources");

    }

    /**
     * Called during the initialization of the event handler to provide the
     * bean id to the event handler.
     * Subclasses should implement this to perform any setup of initial
     * information needed to handle the user interaction with the content
     * info objects that this task's associated view provides.
     *
     * @param beanid unique id of the bean this handler is for.
     */
    public void initializeHandler( long beanid )
    {
        beanID = beanid;
    }

    /**
     * Called by the console in response to a user requesting a popup menu
     * for a specific content info object.  Subclasses should implement this
     * to provide a popup menu based on the object information passed.
     * that this task's associated view provides.
     *
     * @param moid The managed object id (or filter object id) that the content info
     *             object is presented under.
     * @param coid The content info object id that the popup menu is requested for.
     * @return The popup menu that should be displayed for the specified object.
     */
    public JPopupMenu getPopupMenu( long moid, long coid )
    {
        // Create the popup menu to return.
        JPopupMenu popMenu = new JPopupMenu();

        TWGConObject ciobj = TWGConObject.FindObject( coid );

        // We only want to create a popup menu for the PostIt notes - not for
        // the primary PostIt object
        if( (ciobj != null) &&
            (ciobj instanceof TWGConContentInfoObject) &&
            (((TWGConContentInfoObject)ciobj).isPrimaryContentInfo() == false) )
        {
            // Create the Display note menu item and add it to the popup.
            JMenuItem item = new JMenuItem( bcpiResources.getString( "DisplayNoteMenuItem" ));
            item.setActionCommand( "DisplayPostIt" );
            popMenu.add( item );

            // Create the Remove menu item and add it to the popup.
            item = new JMenuItem( bcpiResources.getString( "RemoveNoteMenuItem" ));
            item.setActionCommand( "RemovePostItFromOID" );
            popMenu.add( item );
        }

        return popMenu;
    }

    /**
     * Called when a Command that has been sent completes.  Required to
     * satisfy CommandCompleteListener implementation.
     * @param cmd  The ActionEvent sent for the menu item selected.
     */
    public void CommandComplete( Command cmd )
    {
        if( cmd.ReturnCode() == 0 )
        {
            switch( (int)cmd.CommandCode() )
            {
                case BCPostItConstants.GET_POST_IT_NOTE_TEXT_CMD:
                {
                    // This is simply a stand-in for purposes of
                    // demonstration.  A real application would provide
                    // a display frame using the PostIt icon.
                    JOptionPane.showMessageDialog( null,
                                                   ((BCGetPostItNoteTextCmd)cmd).getPostItNoteText(),
                                                   bcpiResources.getString( "PostItMessagePmt" ),
                                                   JOptionPane.PLAIN_MESSAGE );
                    break;
                }
            }
        }
    }

    /**
     * Called by the console in response to a user selecting one of the items in
     * the popup menu provided.  Subclasses should implement this to provide the
     * action code they wish to execute for the popups they created.
     *
     * @param evt  The ActionEvent sent for the menu item selected.
     * @param moid The managed object id (or filter object id) that the content info
     *             object is presented under.
     * @param coid The content info object id that the popup menu was over.
     */
    public void actionPerformed( ActionEvent evt, long[] moid, long[] coid )
    {
        int i;
        System.out.println( "CAROLYN - BCPostItContentInfoEventHandler.actionPerformed" );
        if ( evt.getActionCommand().equals( "RemovePostItFromOID" ) )
        {
            TWGConObject obj = null;

            // Construct the LongValueSet of moids from which these coids should 
            // be removed.
            LongValueSet lvs = new LongValueSet();
            for( i=0; i<moid.length; i++ )
            {
                lvs.InsertValue( moid[i] );
            }

            // Do for each content info object passed in.
            for (i = 0; i < coid.length; i++)
            {
                obj = TWGConObject.FindObject( coid[i] );
                // Make sure this really is a content info object
                if ( obj instanceof TWGConContentInfoObject )
                {
                    // Create a command to the server that will remove the moid
                    // from this note.  
                    BCRemovePostItNoteCmd cmd = new BCRemovePostItNoteCmd( coid[i], lvs );

                    // Send the remove command.
                    try
                    {
                        // Get this task frame's ServiceNode
                        ServiceNode sn = new ServiceNode();
                        if ( sn.Create() )
                        {
                            // Send note create command
                            boolean brc = sn.SendAsynchCommand( cmd );
                            sn.Close();
                        }
                    }
                    catch (BadServiceNodeImplException snix)
                    {
                        System.out.println(snix);
                    }
                    catch (ServiceNodeClosedException sncx)
                    {
                        System.out.println(sncx);
                    }
                    catch (ServiceNodeException snx)
                    {
                        System.out.println(snx);
                    }
                    catch (InterruptedException ix)
                    {
                        System.out.println(ix);
                    }
                }
            }
        }
        else if ( evt.getActionCommand().equals( "DisplayPostIt" ) )
        {
            for (i = 0; i < coid.length; i++)
                doubleClickRequest( moid[i], coid[i] );
        }
    }

    /**
     * Called by the console in response to a user double-clicking on a specific
     * content info object for a specific oid.  Subclasses should implement this
     * to provide any open/edit (if any) object requests supported for the content info
     * object(s) that are associated with this object.
     *
     * @param moid The managed object id (or filter object id) that the content info
     *             object is presented under.
     * @param coid The content info object id that the popup menu is requested for.
     */
    public void doubleClickRequest( long moid, long coid )
    {
        // Open up a small frame and display the contents of the note.
        TWGConObject obj = TWGConObject.FindObject( coid );
        // Make sure this really is a content info object
        if ( obj instanceof TWGConContentInfoObject )
        {
            // Create a command to the server that will remove the moid
            // from this note.  
            BCGetPostItNoteTextCmd cmd = new BCGetPostItNoteTextCmd( obj.ObjectID(), 
                                                                     this );

            // Send the remove command.
            try
            {
                // Get this task frame's ServiceNode
                ServiceNode sn = new ServiceNode();
                if ( sn.Create() )
                {
                    // Send note create command
                    boolean brc = sn.SendAsynchCommand( cmd );
                    sn.Close();
                }
            }
            catch (BadServiceNodeImplException snix)
            {
                System.out.println(snix);
            }
            catch (ServiceNodeClosedException sncx)
            {
                System.out.println(sncx);
            }
            catch (ServiceNodeException snx)
            {
                System.out.println(snx);
            }
            catch (InterruptedException ix)
            {
                System.out.println(ix);
            }
        }
    }

    //******************************************************************
    // Private data members
    //******************************************************************
    private ListResourceBundle bcpiResources;
    private long beanID;
}
