/******************************************************************************
 * Licensed Materials - Property of IBM                                       *
 *                                                                            *
 * PID (5697-TKT)                                                             *
 *                                                                            *
 * (C) Copyright by IBM 2000. All Rights Reserved.                            *
 *****************************************************************************/
package com.BobCo.postit;

import com.tivoli.twg.engine.InvalidObjectIDException;
import com.tivoli.twg.engine.TWGContentInfoObject;
import com.tivoli.twg.engine.TWGContentInfoObjectHandler;
import com.tivoli.twg.engine.TWGObject;
import com.tivoli.twg.libs.BadServiceNodeImplException;
import com.tivoli.twg.libs.IntKeyTable;
import com.tivoli.twg.libs.IntValueSet;
import com.tivoli.twg.libs.LongValueSet;
import com.tivoli.twg.libs.ServiceNode;
import com.tivoli.twg.libs.ServiceNodeClosedException;
import com.tivoli.twg.libs.ServiceNodeException;
import com.tivoli.twg.log.TWGRas;

/*****************************************************************************
*                         Bob's Software House
*                    A Division of BobCo Enterprises
*****************************************************************************/
/****************************************************************************
* This is the base class for populating and maintaining the content info
* objects for a task association's view.  This abstract class provides the
* interface subclasses must implement in order to provide content info when
* requested by the console.  It is an abstract class so that it can loaded
* as a bean (through a default ctor) instead of being defined as an interface.
*****************************************************************************/
public class BCPostItContentInfoObjectHandler extends TWGContentInfoObjectHandler
{
    // PostIt Bean's id
    private long beanID;
    // Primary Content Info Object for PostIt notes.
    private TWGContentInfoObject primaryCIO = null;
    // Maintain this to correlate note ids with object ids
    private IntKeyTable          cioList    = new IntKeyTable();
    // Current view setting
    private boolean              viewActive = false;



    /**
     * Construct the handler object.  Default ctor so that it can be loaded
     * as a bean.
     */
    public BCPostItContentInfoObjectHandler()
    {
        super();
        TWGRas.debug( TWGRas.EX1,  "BCPostItContentInfoObjectHandler.ctor()" );
    }

    /**
     * Called during the initialization of the TWGContentInfoBeans.
     * Subclasses should implement this to perform any setup of initial
     * information needed to create and maintain the content info objects
     * that this task's associated view provides.
     *
     * @param beanid unique id of the bean this handler is for.
     */
    public void initializeContent( long beanid )
    {
        TWGRas.entryExit( TWGRas.EX1,  "BCPostItContentInfoObjectHandler.initializeContent(" + beanid + ")" );
        beanID = beanid;
    }

    /**
     * Called whenever this bean's view is activated or inactivated.
     * When the first console request's console information, this view
     * is activated and this is called with a true value.  When the last
     * console showing this view no longer needs it, then this view is
     * deactivated and this method is called with a false value.
     * This means either no consoles are running, or if any are,
     * the view this bean could provide is not selected by any user.
     *
     * @param viewActive If true, then this handler should provide content
     *        information for its task association view.  If false, then
     *        content information is no longer needed for any console.
     */
    public void activateContentInfo( boolean viewActive )
    {
        TWGRas.entry( TWGRas.EX1,  "BCPostItContentInfoObjectHandler.activateContentInfo("+viewActive+")");
        this.viewActive = viewActive;
        if ( viewActive )
        {
            TWGRas.debug( TWGRas.EX1,  "BCPostItContentInfoObjectHandler.activateContentInfo view active: creating primaryCIO " + primaryCIO );
            // Create a Primary Content Info object which references all of
            // the objects that have reminder notes attached to them.
            LongValueSet moids = getPostItLists();
            // If we haven't already created a primary content info object, do so now.
            if ( primaryCIO == null )
            {
                if (( moids != null ) && ( moids.Length() > 0 ))
                {
                    try
                    {
                        TWGRas.debug( TWGRas.EX1,  "BCPostItContentInfoObjectHandler.activateContentInfo creating primary content info object");
                        primaryCIO = new TWGContentInfoObject( "ReminderNoteViewTitle",
                                                               "com.BobCo.postit.BCPostItResources", 
                                                               beanID, 
                                                               moids, 
                                                               true );
                    }
                    catch ( InvalidObjectIDException ioide )
                    {
                        TWGRas.error( TWGRas.EX1,  "BCPostItContentInfoObjectHandler.activateContentInfo InvalidObjectIDException", ioide );
                    }
                }
            }
        }
        // Otherwise we are turning OFF the views, so get rid of our content info objects.
        else
        {
            TWGRas.debug( TWGRas.EX1,  "BCPostItContentInfoObjectHandler.activateContentInfo view inactive: deleting primaryCIO " + primaryCIO );
            if (primaryCIO != null)
            {
                TWGRas.debug( TWGRas.EX1,  "BCPostItContentInfoObjectHandler.activateContentInfo deleting primary content info object");
                primaryCIO.Delete();
                primaryCIO = null;
            }
        }
        TWGRas.exit( TWGRas.EX1,  "BCPostItContentInfoObjectHandler.activateContentInfo()");
    }

    /**
     * Called to request the bean update it's content information.
     * This method is called in response to an UpdateContentInfoCmd
     * request, allowing tasks to update themselves.
     */
    public void updateContentInfo()
    {
        TWGRas.entry( TWGRas.EX1,  "Enter BCPostItContentInfoObjectHandler.updateContentInfo()" );
        // Check if we have objects to update.  If none, just exit.
        if ( viewActive )
        {
            // If so, then retrieve the list of all systems that have reminder
            // notes attached to them.
            LongValueSet moids = getPostItLists();
            TWGRas.debug( TWGRas.EX1,  "BCPostItContentInfoObjectHandler.updateContentInfo() moids " + moids );

            // If we have a list of systems, then update content info object.
            if (( moids != null ) && ( moids.Length() > 0 ))
            {
                // If there is no primary content info object, create one so it can function
                // as the parent node for the postIt notes
                if ( primaryCIO == null )
                {
                    try
                    {
                        primaryCIO = new TWGContentInfoObject( "ReminderNoteViewTitle",
                                                               "com.BobCo.postit.BCPostItResources", 
                                                               beanID, 
                                                               moids, 
                                                               true );
                    }
                    catch ( InvalidObjectIDException ioide )
                    {
                        TWGRas.error( TWGRas.EX1,  "BCPostItContentInfoObjectHandler InvalidObjectIDException" + ioide );
                    }
                }

                // remove any deleted note objects
                BCPostItNoteCIO ciobj = null;
                Object[] ciobjs = cioList.GetAllObjects();
                for( int i=0; i<ciobjs.length; i++ )
                {
                    ciobj = (BCPostItNoteCIO)ciobjs[i];
                    if( (ciobj != null) && (ciobj.isNoteDeleted() == true) )
                    {
                        TWGRas.debug( TWGRas.EX1,  "BCPostItContentInfoObjectHandler.updateContentInfo Removing PostIt Note from list " + ciobj.getNoteID() );
                        cioList.RemoveObject( ciobj.getNoteID() );
                        ciobj.Delete();
                    }
                }
                
                // Insert all added nodes
                for( int i=0; i<moids.Length(); i++ )
                {
                    long moid = moids.GetValue( i );
                    // Get the specific note information for this system
                    IntValueSet nids = getPostItNotesForObject( moid );

                    // Lets process each note that is attached to this object
                    LongValueSet amoids = null;
                    for (int n=0; n<nids.Length(); n++)
                    {
                        // First check if we have already created a content info object for this note.
                        int noteID = nids.GetValue( n );
                        TWGRas.debug( TWGRas.EX1,  "BCPostItContentInfoObjectHandler.updateContentInfo - check that " + noteID + " has moid " + moid + " in list" );
                        ciobj = (BCPostItNoteCIO)cioList.FindObject( noteID );
                        if ( ciobj == null )
                        {
                            // otherwise if no content info object, then create one.
                            TWGRas.debug( TWGRas.EX1,  "BCPostItContentInfoObjectHandler.updateContentInfo - create new note object for moid " + moid );
                            try
                            {
                                amoids = new LongValueSet();
                                amoids.InsertValue( moid );
                                ciobj = new BCPostItNoteCIO( "ReminderNoteObjectTitle",
                                                             "com.BobCo.postit.BCPostItResources", 
                                                             beanID, 
                                                             amoids, 
                                                             noteID, 
                                                            false );
                                cioList.InsertObject( noteID, ciobj );
                            }
                            catch ( InvalidObjectIDException ioide )
                            {
                                TWGRas.error( TWGRas.EX1,  "BCPostItContentInfoObjectHandler.updateContentInfo - InvalidObjectIDException ", ioide);
                            }
                        }
                        else
                        {
                            TWGRas.debug( TWGRas.EX1,  "BCPostItContentInfoObjectHandler.updateContentInfo - finds note ID " + ciobj.getNoteID() );

                            // Attempt to add the new value, and if so, update the object.
                            amoids = ciobj.getAssociatedObjectIDs();
                            if( amoids == null )
                                amoids = new LongValueSet();
                            if( amoids.InsertValue( moid ) == true )
                            {
                                TWGRas.debug( TWGRas.EX1,  "BCPostItContentInfoObjectHandler.updateContentInfo - moids after is " + moids );
                                ciobj.setAssociatedObjectIDs( amoids );
                            }
                        }
                        TWGRas.debug( TWGRas.EX1,  "BCPostItContentInfoObjectHandler.updateContentInfo - added object " + ciobj.getName() + " for NoteID " + ciobj.getNoteID() );
                    }
                }

                // associate the primaryCIO withthe current set of moids
                primaryCIO.setAssociatedObjectIDs( moids );
            }
            else
            {
                // This primary CIO is no longer needed
                if( primaryCIO != null )
                {
                    TWGRas.debug( TWGRas.EX1,  "BCPostItContentInfoObjectHandler.updateContentInfo deleting primary content info object");
                    primaryCIO.Delete();
                    primaryCIO = null;
                }
            }
        }
        // If none, then get rid of the object.
        else
        {
            if( primaryCIO != null )
            {
                primaryCIO.Delete();
                primaryCIO = null;
            }
        }
        TWGRas.exit( TWGRas.EX1,  "Exit BCPostItContentInfoObjectHandler.updateContentInfo()" );
    }

    /**
     * Called in response to the user expanding one of the primary content info
     * objects for a specific moid
     *
     * @param moid  object id of the object requesting content info
     */
    public void getContentInfo( long moid )
    {
        TWGRas.entryExit( TWGRas.EX1,  "BCPostItContentInfoObjectHandler.getContentInfo("+moid+")");
    }


    /**
     * Convenience method for retrieving the current post it list of objects
     * from the post it task server.
     */
    private LongValueSet getPostItLists()
    {
        TWGRas.entry( TWGRas.EX1,  "BCPostItContentInfoObjectHandler.getPostItList" );
        LongValueSet postItList = null;

        // Need to get the current list of systems that have notes attached.
        BCRetrievePostItListCmd cmd = new BCRetrievePostItListCmd();
        try
        {
            ServiceNode sn = new ServiceNode();
            if ( sn.Create()  )
            {
                // Send retrieve list command
                sn.SendCommand( cmd );
                sn.Close();

                // If command successful, we have list, so proceed
                if ( cmd.ReturnCode() == 0 )
                {
                    postItList = cmd.getPostItList();
                }
            }
        }
        catch (BadServiceNodeImplException snix)
        {
            TWGRas.error( TWGRas.EX1,  "BadServiceNodeImplException", snix );
        }
        catch (ServiceNodeClosedException sncx)
        {
            TWGRas.error( TWGRas.EX1,  "ServiceNodeClosedException", sncx );
        }
        catch (ServiceNodeException snx)
        {
            TWGRas.error( TWGRas.EX1,  "ServiceNodeException", snx );
        }
        catch (InterruptedException ix)
        {
            TWGRas.error( TWGRas.EX1,  "InterruptedException", ix );
        }
        TWGRas.exit( TWGRas.EX1, "BCPostItContentInfoObjectHandler.getPostItList" );

        return postItList;
    }


    /**
     * Convenience method for retrieving the current post it list of objects
     * from the post it task server.
     */
    private IntValueSet getPostItNotesForObject( long oid )
    {
        IntValueSet postItNotes = null;

        // Need to get the current list of systems that have notes attached.
        BCGetPostItNotesForObjectCmd cmd = new BCGetPostItNotesForObjectCmd( oid );
        try
        {
            ServiceNode sn = new ServiceNode();
            if ( sn.Create()  )
            {
                // Send retrieve list command
                sn.SendCommand( cmd );
                sn.Close();

                // If command successful, we have list, so proceed
                if ( cmd.ReturnCode() == 0 )
                    postItNotes = cmd.getObjectsPostItList();
            }
        }
        catch (BadServiceNodeImplException snix)
        {
            TWGRas.error( TWGRas.EX1,  "BCPostItContentInfoObjectHandler.getPostItNotesForObject BadServiceNodeImplException ", snix );
        }
        catch (ServiceNodeClosedException sncx)
        {
            TWGRas.error( TWGRas.EX1,  "BCPostItContentInfoObjectHandler.getPostItNotesForObject ServiceNodeClosedException ", sncx );
        }
        catch (ServiceNodeException snx)
        {
            TWGRas.error( TWGRas.EX1,  "BCPostItContentInfoObjectHandler.getPostItNotesForObject ServiceNodeException ", snx );
        }
        catch (InterruptedException ix)
        {
            TWGRas.error( TWGRas.EX1,  "BCPostItContentInfoObjectHandler.getPostItNotesForObject InterruptedException", ix );
        }
        TWGRas.exit( TWGRas.EX1,  "BCPostItContentInfoObjectHandler.getPostItNotesForObject" );

        return postItNotes;
    }
}
