/******************************************************************************
 * Licensed Materials - Property of IBM                                       *
 *                                                                            *
 * (C) Copyright by IBM 2003.  All Rights Reserved.                           *
 *****************************************************************************/

package com.BobCo.webobject;

// this will need to be fine-tuned
import com.tivoli.twg.engine.InvalidObjectIDException;
import com.tivoli.twg.engine.TWGManagedObject;
import com.tivoli.twg.engine.TWGManagedObjectConstants;
import com.tivoli.twg.engine.TWGManagedObjectLifeCycleListener;
import com.tivoli.twg.engine.TWGManagedObjectStateListener;
import com.tivoli.twg.engine.TWGObjectDestroyException;
//import com.tivoli.twg.engine.TWGTaskServer;
import com.tivoli.twg.libs.ObjectQueue;
import com.tivoli.twg.libs.QueueBatch;
import com.tivoli.twg.log.TWGRas;


/**
 * This class implements the TWGManagedObjectLifeCycleListener and
 * TWGManagedObjectStateListener interfaces to receive messages
 * when new managed objects are created.  It sends a message to
 * another thread using the QueueBatch class where the new managed
 * object is checked to see if it is also a web server.
 *
 * @see com.tivoli.twg.engine.TWGManagedObjectLifeCycleListener
 * @see com.tivoli.twg.engine.TWGManagedObjectStateListener
 * @see com.tivoli.twg.libs.QueueBatch
 */
class BCWebServerLifeCycleListener
    implements TWGManagedObjectLifeCycleListener, TWGManagedObjectStateListener

{
  //use ObjectQueue to hold the commands that we need to process
  private static QueueBatch queue;

  /**
   * Constructor.
   */
  public BCWebServerLifeCycleListener()
  {
    //create the queue that will hold MOs for us to investigate
    queue = new QueueBatch("BCWebServerDiscoveryQueue");
  }

  /**
   * Add a queue element for processing.
   * @param qe The BCWebServerManagedObject to enqueue for processing
   */
  public static void addQueueElement(TWGManagedObject managedObject)
  {
    // Since our discovery expects a list, create a TWGManagedObject
    // array with a length of 1.
    TWGManagedObject[] managedObjectList = new TWGManagedObject[1];
    managedObjectList[0] = managedObject;
    BCWebServerDiscoveryRequest qe = new BCWebServerDiscoveryRequest(managedObjectList);
    queue.enqueue(qe);
  }

  /**
   * Process creation of managed object.
   *
   * @param managedObject managed object created.
   */
  public void managedObjectCreated( TWGManagedObject managedObject )
  {
    TWGRas.entryExit(TWGRas.EX7,
                     "BCWebServerLifeCycleListener.managedObjectCreated(): entry, name = "
                     + managedObject.getName());

    // If the managed object is not one of our own BCWebServerManagedObjects,
    // which we should already know about, then send a message to our
    // discovery queue.
    if (!(managedObject instanceof BCWebServerManagedObject))
    {
      BCWebServerLifeCycleListener.addQueueElement(managedObject);
    }
  }

  /**
   * Process destruction of managed object.
   *
   * @param managedObject managed object destroyed.
   */
  public void managedObjectDestroyed( TWGManagedObject managedObject )
  {
    TWGRas.entryExit(TWGRas.EX7,
                     "BCWebServerLifeCycleListener.managedObjectDestroyed(): entry name = "
                     + managedObject.getName());

    // If the managed object is not one of our own BCWebServerManagedObjects,
    // which we should already know about, then send a message to our
    // discovery queue, which will check if a corresponding web server
    // managed object still exists
    if (!(managedObject instanceof BCWebServerManagedObject))
    {
      BCWebServerLifeCycleListener.addQueueElement(managedObject);
    }
  }

  /**
   * Process change of managed object.
   *
   * @param managedObject managed object that changed.
   * @param reason reason for change notification.
   */
  public void managedObjectChanged( TWGManagedObject managedObject, int reason )
  {
    TWGRas.entryExit(TWGRas.EX7,
                     "BCWebServerLifeCycleListener.managedObjectChanged(): entry, name = "
                     + managedObject.getName() + ", reason = " + reason);

    switch (reason) {
        case TWGManagedObjectConstants.CHANGED_IP_ADDRESS_SET:
        case TWGManagedObjectConstants.CHANGED_NAME:
            // send a message to discover any corresponding web servers
            // if there are any corresponding web servers with the old
            // name or IP address, they'll have to be deleted manually.
            BCWebServerLifeCycleListener.addQueueElement(managedObject);
            break;

        case TWGManagedObjectConstants.CHANGED_CLIENT_SERVICES:
        case TWGManagedObjectConstants.CHANGED_ATTRIBUTE:
        case TWGManagedObjectConstants.CHANGED_CHASSIS_MEMBERS:
        case TWGManagedObjectConstants.CHANGED_ACCESS:
        default:
            break;
    }//switch
  }

  /**
   * Process managed object state change.
   *
   * @param managedObject managed object.
   * @param oldState old state of managed object.
   * @param newState new state of managed object.
   */
  public void managedObjectStateChange( TWGManagedObject managedObject, int oldState, int newState )
  {
    long managedObjectIdentifier = 0;
    String managedObjectName = null;
    BCWebServerManagedObject webserver = null;
    boolean retry = false;

    TWGRas.entryExit(TWGRas.EX7,
                     "BCWebServerLifeCycleListener.managedObjectStateChange(): entry, name = "
                     + managedObject.getName() + ", state = " + managedObject.getState());


    managedObjectName = managedObject.getName();

    if ( oldState != newState )
    {
      switch ( newState )
      {
        case TWGManagedObject.MOSTATE_ERROR_ONLINE:
        case TWGManagedObject.MOSTATE_NORMAL_ONLINE:
        case TWGManagedObject.MOSTATE_INVALID_OBJECT:
        case TWGManagedObject.MOSTATE_UNKNOWN:
        case TWGManagedObject.MOSTATE_INDETERMINATE:
        case TWGManagedObject.MOSTATE_ERROR_OFFLINE:
        case TWGManagedObject.MOSTATE_NORMAL_OFFLINE:
          // If the managed object is not one of our own BCWebServerManagedObjects,
          // which we should already know about, then send a message to our
          // discovery queue, which will find out if the state has changed for any
          // corresponding web server managed objects
          if (!(managedObject instanceof BCWebServerManagedObject))
          {
            BCWebServerLifeCycleListener.addQueueElement(managedObject);
          }
          break;

        // we don't care about this state change
        case TWGManagedObject.MOSTATE_UNLICENSED:
        default:
          break;

      }
    }
  }
}
