/******************************************************************************
 * Licensed Materials - Property of IBM                                       *
 *                                                                            *
 * (C) Copyright by IBM 2003.  All Rights Reserved.                           *
 *****************************************************************************/

package com.BobCo.webobject;

import java.io.File;
import java.io.FilenameFilter;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.NotSerializableException;
import java.io.ObjectStreamException;

import java.net.InetAddress;
import java.net.UnknownHostException;

import java.util.Enumeration;
import java.util.NoSuchElementException;
import java.util.Observable;
import java.util.Observer;
import java.util.Properties;
import java.util.Timer;
import java.util.TimerTask;
import java.util.Vector;

import com.tivoli.twg.engine.TWGTask;
import com.tivoli.twg.engine.TWGTaskActivation;
import com.tivoli.twg.engine.TWGTaskServer;

import com.tivoli.twg.libs.BadServiceNodeImplException;
import com.tivoli.twg.libs.Command;
import com.tivoli.twg.libs.CommandReceivedListener;
import com.tivoli.twg.libs.PropertiesFile;
import com.tivoli.twg.libs.ServiceNode;
import com.tivoli.twg.libs.ServiceNodeException;
import com.tivoli.twg.libs.ServiceNodeClosedException;
import com.tivoli.twg.libs.TWGEnvironment;
import com.tivoli.twg.log.TWGRas;


/**
 * This class implements TWGTaskServer to process BCWebServerManagedObject
 * commands on the server.
 *
 * @see com.tivoli.twg.engine.TWGTaskServer
 * @see com.tivoli.twg.libs.CommandReceivedListener
 */
public class BCWebServerTaskServer implements TWGTaskServer, CommandReceivedListener
{
   /* Constants */
   public static final int WEB_SERVER_CREATION = 14;
   public static final String WEB_SERVER_CLASS = "com.BobCo.webobject.BCWebServerTaskServer";
   public static final int GOOD = 0;
   public static final int BADADDR = 1;
   public static final int BADOTHER = 2;
   public static final int DUPLICATE = 3;

   /* Private Variables */
   private static ServiceNode sn = null;
   private volatile boolean keepGoing = true;
   private boolean DoCallback;

   /**
    * Constructor
    */
   public BCWebServerTaskServer()
   {
     TWGRas.entryExit(TWGRas.EX7, "BCWebServerTaskServer constructor entry");
   }

   /**
    * This method is called prior to starting the thread with this object so
    * that this object can access its TWGTask object.  This method will be
    * called before a thread is started for this object (i.e. before run() is
    * called).
    *
    * @param task  the TWGTask object for this server task.
    */
   public boolean serverActivate(TWGTask task)
   {
      /* Start the service node for the task here */
      boolean rc = true;

      TWGRas.entryExit(TWGRas.EX7, "BCWebServerTaskServer.serverActivate() entry");

      try
      {
         sn = new ServiceNode(WEB_SERVER_CLASS, false);
         sn.RegisterDefaultCommandListener(this);
         if (sn.Create() == false)
         {
            rc = false;
         }
      }
      catch (BadServiceNodeImplException exception)
      {
         rc = false;
      }
      catch (ServiceNodeClosedException exception)
      {
         rc = false;
      }
      catch (ServiceNodeException exception)
      {
         rc = false;
      }

      TWGRas.entryExit(TWGRas.EX7, "BCWebServerTaskServer.serverActivate() exit, rc = " + rc);

      return rc;
   }

   /**
    * Called once the server is deactivating this task
    */
   public void serverDeactivate()
   {
      TWGRas.entryExit(TWGRas.EX7, "BCWebServerTaskServer.serverDeactivate() entry");

      try
      {
         sn.DeregisterDefaultCommandListener();
         try
         {
            sn.Close();
         }
         catch (ServiceNodeException exception) {}
      }
      catch (NullPointerException exception) {}

      TWGRas.entryExit(TWGRas.EX7, "BCWebServerTaskServer.serverDeactivate() exit");
   }

   /**
    * This method is called when a subtask activation has been received from
    * the console for this task.  At a bare minimum, the TWGTaskServer must
    * call act.subtaskActivated() with either TWGTaskConstants.RETURN_OK
    * or TWGTaskConstants.GENERAL_FAILURE to indicate success or failure for
    * activating the subtask.
    *
    * @param act  the TWGTaskActivation object for the subtask being activated.
    */
   public int subtaskActivate(TWGTaskActivation act)
   {
      return TWGTask.SUBTASK_ACT_COMPLETE;
   }

   /**
    * This method is called when a subtask deactivation has been received from
    * the console for this task.
    */
   public void subtaskDeactivate(TWGTaskActivation act) {}

   /**
    * This method is part of being runnable which the TWGTaskServer interface
    * extends.  For this test case, we'll simply sit in a loop until we're told
    * to exit by serverDeactivate().
    */
   public void run()
   {
      Thread ThisThread = Thread.currentThread();
      TWGRas.entryExit(TWGRas.EX7, "BCWebServerTaskServer.run() entry");

      try
      {
         if (ThisThread != null)
         {
            ThisThread.setName("BCWebServerTaskServer");
         }
      }
      catch (SecurityException exception) {}

      while (keepGoing)
      {
         try
         {
           TWGRas.debug(TWGRas.EX7, "BCWebServerTaskServer.run(): calling ProcessCommands()");
           sn.ProcessCommands();
         }
         catch (ServiceNodeClosedException exception)
         {
            keepGoing = false;
            TWGRas.error(TWGRas.EX7, "BCWebServerTaskServer.run(): ServiceNodeClosedException: " + exception);
         }
         catch (ServiceNodeException exception)
         {
            TWGRas.error(TWGRas.EX7, "BCWebServerTaskServer.run(): ServiceNodeException: " + exception);
         }
         catch (InterruptedException exception)
         {
            TWGRas.error(TWGRas.EX7, "BCWebServerTaskServer.run(): InterruptedException: " + exception);
         }
      }
   }

   public static ServiceNode getWebTaskServiceNode() throws ServiceNodeClosedException
   {
     if (sn == null)
     {
       throw new ServiceNodeClosedException();
     }

     return sn;
   }

   public boolean CommandReceived(ServiceNode CallBackSN, Command cmd)
   {
     TWGRas.entryExit(TWGRas.EX7, "BCWebServerTaskServer.CommandReceived() entry, cmd = " + cmd.CommandCode());

     /* What type of command is this here */
     switch ((int)cmd.CommandCode())
     {
        case WEB_SERVER_CREATION:
           /* A request to create a new device from the Console based on user input */
           int retStatus = BADOTHER;

           /* Make sure we received 2 parameters */
           if (cmd.NumInputParms() == 2)
           {
             try
             {
               String hostName = (String)cmd.GetObjectFromInputParm(0);
               String address = (String)cmd.GetObjectFromInputParm(1);

               /* Now try to add the device to the list */
               if ((hostName != null) && (address != null))
               {
                 /* Verify IP address for hostname */
                 InetAddress destAddress = null;
                 String ipAddress = null;
                 try
                 {
                    destAddress = InetAddress.getByName(hostName);
                    ipAddress = destAddress.getHostAddress();
                    TWGRas.debug(TWGRas.EX7, "BCWebServerTaskServer.CommandReceived(): destAddress = " +
                                       destAddress + ", ipAddress = " + ipAddress);
                 }
                 catch (UnknownHostException exception)
                 {
                   TWGRas.error(TWGRas.EX7, "BCWebServerTaskServer.CommandReceived(): Can't resolve IP address for host \"" +
                                      hostName + "\"");
                   retStatus = BADADDR;
                 }
                 if (ipAddress != null)
                 {
                   if (BCWebServerManagedObject.getWebServer(ipAddress) == null)
                   {
                     TWGRas.debug(TWGRas.EX7,
                                  "BCWebServerTaskServer.CommandReceived(): creating new BCWebServerManagedObject: "
                                  + hostName + ", address: " + ipAddress);
                     // create the new webserver managed object
                     BCWebServerManagedObject newWebServer = new BCWebServerManagedObject(hostName, ipAddress);
                     // ping the webserver to set the state correctly.
                     if (newWebServer == null)
                     {
                       retStatus = BADOTHER;
                     }
                     else
                     {
                       newWebServer.startPing();
                       retStatus = GOOD;
                     }
                   }
                   else
                   {
                     retStatus = DUPLICATE;
                   }
                 }
               }
             }
             catch (ObjectStreamException exception) {}
             catch (ClassNotFoundException exception) {}
           }
           try
           {
              cmd.AddObjectAsOutputParm(new Integer(retStatus));
              //cmd.AddObjectAsOutputParm(new Long(moid));
           }
           catch (NotSerializableException exception) {}
           break;
     }
     return true;
   }
}
