package com.BobCo.cli;

import com.tivoli.twg.engine.TWGManagedObject;
import com.tivoli.twg.engine.TWGManagedObjectConstants;
import com.tivoli.twg.libs.TWGStringArray;
import com.tivoli.twg.libs.ServiceNode;
import com.tivoli.twg.libs.BadServiceNodeImplException;
import com.tivoli.twg.libs.ServiceNodeException;
import com.tivoli.twg.engine.cli.ICliBundle;
import java.io.Writer;
import java.io.IOException;
import java.util.*;

/**
 * This command line interface (CLI) example shows how to create a
 * bundle extending the command line framework.  The steps to creating
 * a bundle are:<p>
 *
 * <p>(1) implement the ICliBundle interface
 * <p>(2) use the below method signature for all accessible bundle commands:
 * <li> public int commandName(Locale locale, String[] args, Writer out) throws Exception
 * <p>(3) furnish the help method
 *
 * <p> The name of the method is used as the name of the command and this is not re-definable.
 * <p> When a user executes a command the corresponding method is invoked where:
 *           <li> Locale = client locale
 *           <li> args   = command inputs provided by the user
 *           <li> out    = output stream back to the client command shell
 *
 * <p>This particular bundle example implements two usage/help commands:
 * <li> Help - Displays written help message to the user
 * <li> List -  Shows the command syntax
 *
 * <p>Two functional commands are implemented as well.  One to list all of the subagents
 * for a specified set of systems, and the other to verify a list of service nodes.
 * The commands are below:
 * <li> ListSystemSubAgents
 * <li> CheckServiceNodes
 *
 * @see com.tivoli.twg.engine.cli.ICliBundle
 *
 * @author Rob Warren
 * @version 1.0 12/28/01
 */
public class CliSample implements ICliBundle {

    /**
     * Default and only constructor<p>
     * All CLI bundle extensions must have an empty parameter constructor.
     * The constructor can either be ommitted or defined with no parameters
     */
    public CliSample() {}

    /**
     * Mandated bundle command to provide bundle help
     * @param Locale locale - client locale for NLS output
     * @param String[] args - the command arguments (nothing expected for this command)
     * @param Writer out - output stream back to client
     * @return int - client application exit code
     */
    public int help(Locale locale, String[] args, Writer out) throws Exception {
        out.write("This is the BobCo CLI bundle sample\n");
        out.write("Listed below are the bundle commands:\n");
        out.write("  Help - Shows current help screen\n");
        out.write("  List - List the bundle commands and syntax\n");
        out.write("  ListSystemSubAgents <SystemName(1)>...<SystemName(N)> - List subagents on specified system\n");
        out.write("  CheckServiceNodes <IPCNodeName(1)>...<IPCNodeName(N)>  - Check for ServiceNodes active\n");
        return ICliBundle.USAGE;
    }

    /**
     * Bundle command to provide bundle a short/quick command reference
     * @param Locale locale - client locale for NLS output
     * @param String[] args - the command arguments (nothing expected for this command)
     * @param Writer out - output stream back to client
     * @return int - client application exit code
     */
    public int list(Locale locale, String[] args, Writer out) throws Exception {
        out.write("Command list\n");
        out.write("  Help\n");
        out.write("  List\n");
        out.write("  ListSystemSubAgents <SystemName(1)>...<SystemName(N)>\n");
        out.write("  CheckServiceNodes <IPCNodeName(1)>...<IPCNodeName(N)>\n");
        return ICliBundle.USAGE;
    }

    /**
     * List the subagents on the given systems
     * @param Locale locale - client locale for NLS output
     * @param String[] args - the command arguments (contains a list of system names)
     * @param Writer out - output stream back to client
     * @return int - client application exit code
     */
    public int listSystemSubAgents(Locale locale, String[] args, Writer out) throws Exception {

        if(args.length == 0) {
            out.write("No system names specified");
            return ICliBundle.USAGE;
        }

        TWGManagedObject[] mos = TWGManagedObject.getAllManagedObjects();
        for (int i = 0; i < args.length; i++) {
            int foundIdx = -1;
            for (int j = 0; j < mos.length; j++) {
                if (args[i].toUpperCase().equals( mos[j].getName().toUpperCase() )) {
                    foundIdx = j;
                    break;
                }
            }

            // Display agents if system found
            if (foundIdx >= 0) {
                out.write(mos[foundIdx].getName() + "\n");
                TWGStringArray val =
                (TWGStringArray)mos[foundIdx].getAttributeValue(TWGManagedObjectConstants.ATTRIB_CLIENT_SVCS, locale);
                for (int k = 0; k < val.getLength(); k++) {
                    out.write("  " + val.getValueAt(k) + "\n");
                }
            } else {
                out.write(args[i] + " Not Found\n");
            }
        }
        return ICliBundle.OK;
    }

    /**
     * Check for the existance of the named IPC service nodes
     * @param Locale locale - client locale for NLS output
     * @param String[] args - the command arguments (list of ipc ServiceNode names)
     * @param Writer out - output stream back to client
     * @return int - client application exit code
     */
    public int checkServiceNodes(Locale locale, String[] args, Writer out) throws Exception {

        if(args.length == 0) {
            out.write("A service node must be specified");
            return ICliBundle.USAGE;
        }

        for (int i = 0; i < args.length; i++) {
            try {
                if( ServiceNode.ConnectionActive(args[i]) ) {
                    out.write("ServiceNode " + args[i] + " active\n");
                }
                else {
                    out.write("ServiceNode " + args[i] + " not found\n");
                }
            }
            catch (BadServiceNodeImplException bsn) {
                out.write("ServiceNode " + args[i] + " contains errors\n");
            }
            catch (ServiceNodeException sne) {
                out.write("Error checking node " + args[i] + "\n");
            }
        }

        return ICliBundle.OK;
    }
}
