/******************************************************************************
 * Licensed Materials - Property of IBM                            *
 *                                                                            *
 * PID (5697-TKT)                                                             *
 *                                                                            *
 * (C) Copyright by IBM 2000. All Rights Reserved. *
 *****************************************************************************/
package com.BobCo.procman;

import com.tivoli.twg.libs.Command;
import com.tivoli.twg.libs.CommandCompleteListener;
import com.tivoli.twg.libs.IntelByteBuffer;
import com.tivoli.twg.libs.ObjectQueue;
import com.tivoli.twg.libs.ServiceNode;
import com.tivoli.twg.libs.ServiceNodeClosedException;
import com.tivoli.twg.libs.ServiceNodeException;
import com.tivoli.twg.libs.ServiceNodeImplFactory;
import com.tivoli.twg.libs.ServiceNodeLocalImplFactory;

import com.tivoli.twg.procman.api.ExecuteCommandAPI;
import com.tivoli.twg.procman.TWGProcManConstants;


/**
* ExecuteCommandSample demonstrates one way of executing commands on
* agents. This sample shows how to asynchronously start notepad on
* a user specified machine.
*/
public class ExecuteCommandSample
implements CommandCompleteListener
{
   /* Service node instance */
   private ServiceNode sn = null;

   /* Used to indicate when the main method should exit loop */
   private boolean keepRunning = true;

   /* Queue for CommandComplete listener to indicate next action. */
   private ObjectQueue workQueue = null;

   /* Constants used to indicate if the request is for starting or stopping the ProcMgr service */
   private static final int SVCMGR_START_SVC  = 0x0101;
   private static final int SVCMGR_STOP_SVC   = 0x0102;

   /* TCPIP dotted decimal addr. indicating where to execute command */
   private String destination;


   /**
   * Main method. Creates a service node, starts the ProcMgr service, and 
   * processes requests off the workQueue.
   */
   public static void main(String[] args)
   { 
      // If no arguements provided, display usage message
      if (args.length == 0)
      {
         DisplayUsage();
         return;
      }
   
      Object currentCmd = null;
      Integer    cmdID;
      ExecuteCommandSample ect = new ExecuteCommandSample();

      ect.destination = args[0];

      // Create a service node
      ect.serviceNodeInit();

      // Start the ProgMgr service on specified agent
      if (!ect.startProcMgrService())
      {
         System.out.println("Error, could not start ProcMgr service on agent");
         ect.keepRunning = false;
      }


      while (ect.keepRunning)
      {
         try
         {   
            // Pull a command off the work queue or wait until
            // something gets put on the queue
            currentCmd = ect.workQueue.dequeue();
         }
         catch (InterruptedException e)
         {
            System.out.println("Error interrupted exception");
         }

         // Take action based on the command
         cmdID = (Integer)currentCmd;
         switch (cmdID.intValue())
         {
            case 1:           // shutdown
               ect.keepRunning = false;
               break;
            case 2:           // build and send command       
               ect.buildAndSendExecuteCommand();
         } 
      } // end while

      System.out.println("Shutting down");

      // Stop ProcMgr service on agent
      ect.stopProcMgrService();

      // Close down the service node
      ect.serviceNodeClose();
   } // end main


   /**
   * Output to screen usage info
   */
   public static void DisplayUsage()
   {
      System.out.println(
         "\nUsage: java ExecuteCommandSample [ip address]\n" +
         "       Where the ip address is a dotted decimal TCPIP address where the command\n" +
         "       is to be executed.");
   }


   /**
   * Constructor - Set up work queue to processes requests
   */
   public ExecuteCommandSample()
   {
      workQueue = new ObjectQueue();
   } // end ExecuteCommandSample


   /**
   * Create service node
   */
   public void serviceNodeInit()
   {
      try
      {
         Class cls = Class.forName("com.tivoli.twg.libs.ServiceNodeLocalImplFactory");
         try
         {
            ServiceNodeImplFactory fct = (ServiceNodeImplFactory)cls.newInstance();
            ServiceNode.SetServiceNodeFactory(fct);
            
            sn = new ServiceNode("ExecuteCmdTest");
            
            if(sn.Create())
               System.out.println("ServiceNode created");
            else
            {
               System.out.println("ERROR: Failed to create ServiceNode");
               return; 
            }
         }
         catch (InstantiationException e)
         {
            System.out.println("ERROR: Instantiation exception");
         }
         catch (IllegalAccessException e)
         {
            System.out.println("ERROR: Illegal access error");
         }
         catch(ServiceNodeException e)
         {
            System.out.println("ERROR: ServiceNode error");
         }
      }
      catch (ClassNotFoundException e)
      {
         System.out.println("ERROR: Class load error, class not found");
      }

   } // end serviceNodeInit


   /**
   * Close service node
   */
   public void serviceNodeClose()
   {
      try
      {
         sn.Close();
      }
      catch (ServiceNodeException e)
      {
         System.out.println("ERROR: ServiceNodeException on close");
      }
   } // end serviceNodeClose


   /**
   * Start process manager agent service on the specified machine.
   * This sample only uses TCPIP, but other protocols could be used.
   * An example of the destination is TCPIP::146.84.86.21::ProcMgr.
   */
   public boolean startProcMgrService()
   {
      System.out.println("Starting local procMgr agent");

      Command cmd = new Command(SVCMGR_START_SVC);
      byte[] p = new byte[IntelByteBuffer.GetASCIIZLength("ProcMgr")]; 
      IntelByteBuffer.WriteASCIIZ(p, "ProcMgr", 0);
      cmd.AttachInputParm(p);
      cmd.SetDestinationAddress("TCPIP::" + destination + "::SvcMgr");
      cmd.setCommandCompleteListener(this);

      try
      {
         sn.SendAsynchCommand(cmd);
      }
      catch (ServiceNodeClosedException e)
      {
         System.out.println("Error: Service node closed exception");
         return false;
      }
      catch (InterruptedException e)
      {
         System.out.println("Error: Interrupted exception");
         return false;
      }
      catch(ServiceNodeException e)
      {
         System.out.println("Error: Service node exception");
         return false;
      }

      System.out.println("Results from starting procmgr are cmd.rc = " + cmd.ReturnCode());
      return true;
   }


   /**
   * Stop ProcMgr service on specified agent
   */
   public void stopProcMgrService()
   {
      System.out.println("Stopping local procMgr agent");

      Command cmd = new Command(SVCMGR_STOP_SVC);
      byte[] p = new byte[IntelByteBuffer.GetASCIIZLength("ProcMgr")];
      IntelByteBuffer.WriteASCIIZ(p, "ProcMgr", 0);
      cmd.AttachInputParm(p);
      cmd.SetDestinationAddress("TCPIP::" + destination + "::SvcMgr");

      try
      {
         sn.SendAsynchCommand(cmd);
      }
      catch (ServiceNodeClosedException e)
      {
         System.out.println("Error: Service node closed exception");
      }
      catch (InterruptedException e)
      {
         System.out.println("Error: Interrupted exception");
      }
      catch(ServiceNodeException e)
      {
         System.out.println("Error: Service node exception");
      }

      System.out.println("Results from stopping procmgr are cmd.rc = " + cmd.ReturnCode());

   } // end stopProcMgrService

   
   /**
   * Build the command to be executed and then execute it on the specified
   * agent.
   */
   public void buildAndSendExecuteCommand()
   {
      System.out.println("Entering buildAndSendExecuteCommand()");

      ExecuteCommandAPI exCmd = new ExecuteCommandAPI();

      exCmd.SetDestinationAddress("TCPIP::" + destination + "::ProcMgr");

      // Alternatively you can use the API method setDestinationFromMoid. This
      // will query the server for the preferred path to the agent and then
      // the method will call SetDestinationAddress.
      //exCmd.setDestinationFromMoid(aMoid, sn); 

      System.out.println("Destination addr = " + exCmd.DestinationAddress());

      // Specify the command to execute asynchrounously on the agent and
      // do not capture the output. 
      exCmd.setAsynch(true);
      exCmd.setCapture(false);  

      // If you want to capture output, setAsynch(false) and setCapture(true).
      // Also you would want to set the timeout period to wait for the command
      // to finish. Below is a sample timeout and command (for NT).
      //exCmd.setTimeout(30); 
      //exCmd.setCommandLine("cmd /c dir c:\");

      // Have notepad be started on the specified agent. NOTE: the application
      // will start minimized.
      exCmd.setCommandLine("notepad.exe");
      exCmd.setCommandCompleteListener(this);

      // Now that all of the parameters have been set. Actually build the command
      // to be sent via the service node
      if (!exCmd.buildCommandLine())
      {
         System.out.println("Error buildCommandLine failed, not all parameters have been set");
      }

      System.out.println("Sending command to agent");
      try
      {
         if (!sn.SendAsynchCommand(exCmd))
         {
            System.out.println("Error: SendAsyncCommand failed");
         }
      }
      catch (ServiceNodeClosedException e)
      {
         System.out.println("Error: Service node closed exception");
      }
      catch (InterruptedException e)
      {
         System.out.println("Error: Interrupted exception");
      }
      catch(ServiceNodeException e)
      {
         System.out.println("Error: Service node exception");
      }

      System.out.println("cmd.rc = " + exCmd.ReturnCode());
   } // end buildAndSendExecuteCommand


   /**
   * Command complete method which overrides the method in the Command class.
   * Based on what completed, put the next action on the work queue. The main
   * method will then wake up and get the item off the queue and call the 
   * appropriate routine.
   */
   public void CommandComplete(Command cmd)
   { 
      ExecuteCommandAPI exCmd;

      // ProcMgr service start completed
      if (cmd.CommandCode() == SVCMGR_START_SVC)
      {
         System.out.println("Command from SVCMGR_START_SVC, return code = " + cmd.ReturnCode());

         if (cmd.ReturnCode() == 0)
         {
            // Put on queue that the next action is to build and send command.
            workQueue.enqueue(new Integer(2));
            
         }  
         else
         {
            System.out.println("Error starting ProcMgr Service");
            // Put on queue that the next action is to shutdown
            workQueue.enqueue(new Integer(1));
         }
      }
      else if (cmd instanceof ExecuteCommandAPI)    
      {
         // Execute command completed
         System.out.println("Command is result of Execute Command completion");
         
         exCmd = (ExecuteCommandAPI)cmd;

         // If everything worked OK, print out the actual return code from the command.
         if (cmd.ReturnCode() == 0)
         {
            System.out.println("Command executed, rc=" + Long.toHexString(exCmd.getExecRC()));

            // If the command returned output this is where you would get the data and
            // do something with it.
            //byte[] cmdOutput = cmd.getOutput();
         }
         else
         {
            System.out.println("Command execute failed, rc = " +
                              Long.toHexString(cmd.ReturnCode()));

            switch((int)cmd.ReturnCode())
            {
               case ((int)Command.CMDRET_SEND_TIMEOUT):
               {
                  System.out.println("Error, command timed out");
                  break;
               }
               case((int)TWGProcManConstants.EXE_NOT_FOUND):
               {
                  System.out.println("Error, executable not found");
                  break;
               }
               case ((int)TWGProcManConstants.EXE_ACCESS_DENIED):
               {
                  System.out.println("Error, access denied to executable");
                  break;
               }            
               case ((int) TWGProcManConstants. EXE_NO_MEM):
               {
                  System.out.println("Error, out of memory");
                  break;
               }
               case ((int) TWGProcManConstants. EXE_TOO_MANY_FILES):
               {
                  System.out.println("Error, too many files opened");
                  break;
               }
               case ((int) TWGProcManConstants. EXE_BAD_FORMAT):
               {
                  System.out.println("Error, bad format for executable");
                  break;
               }
               case ((int) TWGProcManConstants. EXE_SHARE_VIOLATION):
               {
                  System.out.println("Error, sharing violation");
                  break;
               }
               case ((int) TWGProcManConstants. EXE_DYNALINK_ERROR):
               {
                  System.out.println("Error, problem with required DLL"); 
                  break;
               }
               case ((int) TWGProcManConstants.EXE_NO_ANON_CMD_EXEC):
               {
                  System.out.println("Error, no anonymous command execution allowed on agent, must supply userid and password");
                  break;
               }
               default:
               {
                  System.out.println("Error");
               }
            } // end switch
         } // end if

         // Execute command completed, shutdown by putting shutdown command on queue
         workQueue.enqueue(new Integer(1));
      } // end if instance of executeCommand
   } // end CommandComplete
   
} // end class ExecuteCommandSample


