package com.tivoli.twg.mscs;

import com.tivoli.twg.libs.*;
import com.tivoli.twg.engine.*;
import java.util.MissingResourceException;
import java.util.Locale;
/**
 * Implementation of address entry shadow for MSCSManagedObject.
 *
 * @author Mike Primm
 * @version 1.1 - 6/8/05
 */
public class MSCSAddressEntryShadow implements AddressEntryShadow {
    /* Our string bundle                                                    */
    private static MultiLocaleBundle nls = new
        MultiLocaleBundle("com.tivoli.twg.mscs.MSCSNLS");
    /* Best IPC path string                                                 */
    private String ipcpath;
    /* Attribute symbol list                                                */
    private static final String[] attrib_ids = { "ipcpath" };
    /** Our record magic number                                             */
    public static final int MAGIC_NUMBER = 0xD9F3;
    /**
     * Default constructor
     */
    public MSCSAddressEntryShadow() {
        ipcpath = null;
    }
    /**
     * Deliver address entry record to initialize AddressEntryShadow
     * data content
     * @param buf - buffer containing address entry record
     * @param start - offset of start of record in buffer
     * @param len - length of record in buffer
     * @exception AddressEntryFormatException if improper address record
     */
    public void initAddressEntry(byte[] buf, int start, int len)
        throws AddressEntryFormatException {
        /* Check magic number                                               */
        int magic_num = IntelByteBuffer.ReadUSHORT(buf, start +
            IntelByteBuffer.SIZEOF_USHORT);
        if(magic_num != MAGIC_NUMBER)
            throw new AddressEntryFormatException("Bad magic number");
        ipcpath = IntelByteBuffer.ReadCompUnicode(buf, start +
            IntelByteBuffer.SIZEOF_USHORT * 2);
        if(ipcpath.length() == 0)
            ipcpath = null;
    }
    /**
     * Get symbolic names for all attributes supported
     * @return string array of address attribute symbol names
     */
    public String[] getAddressAttributeIDs() {
        return attrib_ids;
    }
    /**
     * Get string representation of attribute with given symbolic name
     * @param symname - symbolic name of attribute to request
     * @return string value of attribute (null if no value)
     */
    public String getAddressAttribute(String symname) {
        if(symname.equals(attrib_ids[0])) {
            return getIPCPath();
        }
        return null;
    }
    /**
     * Get label for attribute with given symbolic name
     * @param symname - symbolic name of attribute to get label
     * @param loc - locale to use for label information
     * @return label string
     */
    public String getAddressAttributeLabel(String symname,
        Locale loc) {
        String name = null;
        try {
            if(symname.equals(attrib_ids[0])) {
                name = "addressshadow.ipcpath.name";
                return nls.getString(loc, name);
            }
        } catch (MissingResourceException mrx) { }
        return name;
    }
    /**
     * Get IPC path for native client
     * @return IPC path string, or null if not currently available
     */
    public String getIPCPath() { return ipcpath; }
    /**
     * Build address entry record using given parameters
     * @param ipcpath - IPC path
     */
    public static byte[] buildAddressEntryRecord(String ipcpath) {
        /* Compute length needed for block                                  */
        int len = IntelByteBuffer.SIZEOF_USHORT +
            IntelByteBuffer.SIZEOF_USHORT +
            IntelByteBuffer.GetCompUnicodeLength(ipcpath);
        /* Allocate buffer                                                  */
        byte[] buf = new byte[len];
        /* Fill in required fields                                          */
        IntelByteBuffer.WriteUSHORT(buf, len, 0);
        IntelByteBuffer.WriteUSHORT(buf,
            MSCSAddressEntryShadow.MAGIC_NUMBER,
            IntelByteBuffer.SIZEOF_USHORT);
        /* Fill in our data                                                 */
        IntelByteBuffer.WriteCompUnicode(buf, ipcpath,
            2*IntelByteBuffer.SIZEOF_USHORT);
        return buf;
    } 
}


