package com.tivoli.twg.mscs;

import com.tivoli.twg.engine.cluster.*;
import com.tivoli.twg.engine.*;
import com.tivoli.twg.log.TWGRas;
import com.tivoli.twg.log.TWGOutput;
import com.tivoli.twg.libs.*;

/**
 * Microsoft Cluster Services managed object factory.
 * @author Mike Primm
 * @version 1.1 - 6/8/05
 */
public class MSCSManagedObjectFactory extends TWGManagedObjectFactory 
    implements TWGServerCommandHandler {
    /** Name of TWGManagedObject class supported by factory                                 */
    public static final String MO_CLASS =
        "com.tivoli.twg.mscs.MSCSManagedObject";
    /** Name of NLS resource used for Cluster managed object support                        */
    public static final String NLSCLASS =
        "com.tivoli.twg.mscs.MSCSNLS";
    /* Name of our address entry shadow class                               */
    private static final String ADDRESSSHADOWCLASS =
        "com.tivoli.twg.mscs.MSCSAddressEntryShadow";
    /* Required license product ID : ITD V2.x clients                                       */
    private static final int LICENSE_PROD_ID = 1;
    /** Name ID for cluster managed objects                                                 */
    private static final String MONAME = "mscsmof.name";
    /** Service manager command to start a given service                    */
    //private static final int SVCMGR_START_SVC = 0x0101;
    /** Service manager command to stop a given service                     */
    //private static final int SVCMGR_STOP_SVC = 0x0102;

    private static final String MSCSAGT_CLIENT_SERVICE = "NativeMO." + MSCSAgentCommand.CLUSTAGT_SVCNODE_NAME;

    /* image set used for this type of device */
    private static final String[] IMAGE_SET = {
        "/com/tivoli/twg/engine/images/clus32ind.gif",
        "/com/tivoli/twg/engine/images/clus32on.gif",
        "/com/tivoli/twg/engine/images/clus32of.gif",
        "/com/tivoli/twg/engine/images/clus32one.gif",
        "/com/tivoli/twg/engine/images/clus32ofe.gif",
        "/com/tivoli/twg/engine/images/unldev32.gif",
        "/com/tivoli/twg/engine/images/clus16ind.gif",
        "/com/tivoli/twg/engine/images/clus16on.gif",
        "/com/tivoli/twg/engine/images/clus16of.gif",
        "/com/tivoli/twg/engine/images/clus16one.gif",
        "/com/tivoli/twg/engine/images/clus16ofe.gif",
        "/com/tivoli/twg/engine/images/unldev16.gif",
    };

    private static final String[] FILTER_SET = {
        "/com/tivoli/twg/engine/images/clus16on.gif",
        "/com/tivoli/twg/engine/images/clus32on.gif"
    };

    /* Reference to MSCSManagedObjectFactory instance                                       */
    private static MSCSManagedObjectFactory factory = null;

    /**
     * Default constructor
     * @exception ClassNotFoundException if MO class not loaded
     */
    public MSCSManagedObjectFactory() throws
        ClassNotFoundException {
        this(MO_CLASS);
    }
    /**
     * Constructor
     * @param managedObjectClassName - TWGManagedObject class name associated with factory
     * @exception ClassNotFoundException if MO class not loaded
     */
    public MSCSManagedObjectFactory(String managedObjectClassName) throws
        ClassNotFoundException {
        super(managedObjectClassName);
        factory = this;
        setName(MONAME, NLSCLASS);      /* Set class name                                   */
        setLicenseProductCode(LICENSE_PROD_ID);	/* Set license key required                 */
        setPingType(TWGPing.PING_TYPE_SERVER_DRIVES_INDIVIDUAL);
        setAddressEntryShadow(ADDRESSSHADOWCLASS);      /* Set addr shadow  */
        /* Register command handlers                                        */
        TWGServerServiceNode.RegisterServerCommandHandler(
            MSCSAddObjectCmd.CMDNAME, this);
        /* Object adds from console supported                               */
        setAddObjectClassname("com.tivoli.twg.mscs.MSCSNewObjectPanel");
        try {
            TWGImageSet Temp = TWGImageSet.FindImageSet(IMAGE_SET);
            TWGImageSet Temp2 = TWGImageSet.FindImageSet(FILTER_SET);
            setImageSet(Temp.PersistID());
            setFilterImageSet(Temp2.ObjectID());
        } catch(InvalidObjectIDException Err) {
        } catch(TWGPersistentObjectSaveException Err) {
        }
    }
    /**
     * Delete method
     */
    public void Delete() {
        /* Unregister command handlers                                      */
        TWGServerServiceNode.UnregisterServerCommandHandler(
            MSCSAddObjectCmd.CMDNAME);
        /* Pass to parent                                                                   */
        super.Delete();
    }
    /**
     * Get instance of cluster managed object factory
     * @return MSCSManagedObjectFactory instance
     */
    public static MSCSManagedObjectFactory GetFactory() {
        return factory;
    }
    /**
     * Test a managed object instance for inclusion into a managed object
     * factory class. This method is used during managed object morphing.
     * It is used by TWGEngine processes to determine the lowest level
     * managed object factory class to which a managed object instance belongs.
     * This method should be implemented by all managed object factory
     * extension classes.
     * @param mofInstanceOf Reference to TWGMofInstanceOf callback interface object.
     * @param mo Reference to managed object instance to be tested.
     * @return byte see TWGManagedObjectFactory.
     */
    public byte instanceOf(TWGMofInstanceOf mofInstanceOf, TWGManagedObject managedObject) {
        if (managedObject instanceof MSCSManagedObject) {
            return TWGManagedObjectFactory.INSTANCE_OF_YES;
        }
        else {
            return TWGManagedObjectFactory.INSTANCE_OF_NO;
        }
    }
    /**
     * Activate the topology processing.
     * @exception ManagedObjectFactoryActivateException if error
     */
    public void Activate() throws ManagedObjectFactoryActivateException {
        super.Activate();
    }
    /**
     * Deactivate the topology processing
     */
    public void Deactivate() {
    }
    /**
     * Start a discovery pass: operation should return quickly, with
     * actual discovery logic not being completed before the method
     * returns.  The default method does nothing.
     */
    public void StartDiscoveryPass() {
    }

    /**
     * Start cluster validation using given native managed object
     * @param nmo - MSCS agent supporting managed object
     * @param clus - Cluster containing object (null if not in cluster)
     */
    static void startClusterValidation(TWGNativeManagedObject nmo,
        MSCSManagedObject clus) {
        new ValidateClusterInfo(nmo, clus);
    }

    private static class ClientServiceSubscriber implements TWGClientServiceListener,
        TWGManagedObjectStateListener {
        public ClientServiceSubscriber() {
        }
        /**
         * Service add notification : indicates that the given client service
         * has been added to the given managed object
         * @param svcname - client service name
         * @param mo - managed object
         */
        public void clientServiceAdded(String svcname,
            TWGManagedObject mo) {
            //TWGOutput.println("Found MSCSAGT system - " + mo.getName());
            if(mo instanceof TWGNativeManagedObject) {
                TWGNativeManagedObject nmo = (TWGNativeManagedObject)mo;
                if(nmo.getPreferredPath() != null)
                    new ValidateClusterInfo(nmo, null);
            }
        }
        /**
         * Service remove notification ; indicates that the given client
         * service has been remove from the given managed object
         * @param svcname - client service name
         * @param mo - managed object
         */
        public void clientServiceRemoved(String svcname,
            TWGManagedObject mo) {
            //TWGOutput.println("MSCSAGT removed from system - " + mo.getName());
        }
        /**
         * Notificaton method for TWGManagedObject state changes.  This
         * method is called after the state has been changed, and provides
         * the previous state value. Implementations of this callback must
         * execute quickly without blocking.
         * @param twgmo - Reference to TWGManagedObject which changed state
         * @param oldstate - Previous state of TWGManagedObject instance
         * @param newstate - New state of TWGManagedObject instance
         */
        public void managedObjectStateChange(TWGManagedObject twgmo,
            int oldstate, int newstate) {
            /* If not native managed object, quit                           */
            if(!(twgmo instanceof TWGNativeManagedObject))
                return;
            /* See if its an object with the MSCSAGT service                */
            TWGNativeManagedObject nmo = (TWGNativeManagedObject)twgmo;
            if(nmo.IsClientServiceSupported(MSCSAGT_CLIENT_SERVICE) == false) {
                return;
            }
            try {
                int id = twgmo.PersistID();
                /* See if object is in any clusters                         */
                IntValueSet clus_ids = 
                    TWGClusterManagedObjectFactory.getClustersForMOID(id);
                /* If so, validate those clusters                           */
                if((clus_ids != null) && (clus_ids.Length() > 0)) {
                    for(int i = 0; i < clus_ids.Length(); i++) {
                        TWGManagedObject clus_mo = 
                            TWGManagedObject.getManagedObjectByID(
                            clus_ids.GetValue(i));
                        if((clus_mo != null) && (clus_mo instanceof MSCSManagedObject)) {
                            clus_mo.startPing();
                        }
                    }
                } else {
		    /* Else, not yet in cluster : validate if online        */
                    if((newstate == TWGManagedObject.MOSTATE_NORMAL_ONLINE) ||
                        (newstate == TWGManagedObject.MOSTATE_ERROR_ONLINE)) {
                        new ValidateClusterInfo(nmo, null);
                    }
                }
            } catch (InvalidObjectIDException ioidx) {
            }
        }
    }

    /** Listener for remote MPM site subscribers                            */
    private static ClientServiceSubscriber listener = null;

    /**
     * Initialize client service subscriber
     */
    public static void initializeServiceSubscriber() {
        listener = new ClientServiceSubscriber();       /* Create listener  */
        /* Register listener                                                */
        TWGManagedObject.registerClientServiceSubscriber(
            MSCSAGT_CLIENT_SERVICE, listener);
        TWGManagedObject.RegisterStateListener(listener);
    }
    /**
     * Terminate client service subscriber
     */
    public static void terminateServiceSubscriber() {
        /* Unregister listener                                              */
        if(listener != null) {
            TWGManagedObject.UnregisterStateListener(listener);
            TWGManagedObject.unregisterClientServiceSubscriber(
                MSCSAGT_CLIENT_SERVICE, listener);
        }
        listener = null;
    }
    /**
     * Inner class for tracking cluster information validation
     */
    public static class ValidateClusterInfo implements CommandCompleteListener,
        ClientServiceListener {
        private TWGNativeManagedObject obj;
        private MSCSManagedObject cluster;
        public ValidateClusterInfo(TWGNativeManagedObject nmo,
            MSCSManagedObject clus) {
            obj = nmo;
            cluster = clus;
            nmo.ClientServiceStart(MSCSAGT_CLIENT_SERVICE, this);
        }
        /**
         * Command completion callback.  This listener method is called when a Command instance
         * registered to this listener is completed.
         * @param cmd - Command instance of command which has completed
         */
        public void CommandComplete(Command cmd) {
            if(cmd instanceof MSCSGetFullConfigCommand) {
                MSCSGetFullConfigCommand cfg = (MSCSGetFullConfigCommand)cmd;
                /* If successful, process results                                           */
                if(cfg.ReturnCode() == 0) {
                    /* Find existing cluster, if defined                                    */
                    String resid = cfg.getClusterNameResource();
                    MSCSManagedObject mo = null;
                    if(resid.length() > 0) {
                        mo = MSCSManagedObject.findMSCSObjectByNameResID(resid);
                    }
                    if(mo == null) {
                        mo = MSCSManagedObject.findMSCSObjectByName(cfg.getClusterName());
                    }
                    boolean was_added = false;
                    try {
                        if(mo == null) {	/* If not found, create new cluster         */
                            IntValueSet init_ids = new IntValueSet();
                            init_ids.InsertValue(obj.PersistID());
                            mo = new MSCSManagedObject(cfg.getClusterName(), 
                                resid, init_ids);
                            mo.save();
                            was_added = true;
                        } else {
                            was_added = mo.addNewMember(obj);
                        }
                        mo.setNodeGood(obj);	/* Mark node as good node                   */
                        mo.updateMSCSCluster(cfg);	/* Update cluster state             */
                        mo.save();
                        if(was_added) {
                            mo.ObjectChangedNotify(TWGManagedObjectConstants.CHANGED_ATTRIBUTE);
                            TWGActiveConsole.objectUpdated(obj);
                            TWGActiveConsole.objectUpdated(mo);
                        }
                    } catch (TWGPersistentObjectSaveException posx) {
                        TWGRas.error(TWGRas.ENGINE, "Error saving MSCSManagedObject");
                    } catch (InvalidObjectIDException ioidx) {
                        TWGRas.error(TWGRas.ENGINE, "Object obsolete");
                    }
                } else {
                    if(cluster != null) {
                        cluster.setNodeBad(obj);	/* Mark node as bad node            */
                    }
                }
            }
            /* Send stop agent request                                                      */
            obj.ClientServiceStop(MSCSAGT_CLIENT_SERVICE, this);
        }
        /**
         * Notification method for completion of StartClientService() request
         * @param mo - TWGManagedObject which request was related to
         * @param resultcode - result code of the request
         */
        public void ClientServiceStartCompleted(TWGManagedObject mo,
            int resultcode) {
            /* If agent started, send request for cluster configuration information         */
            if(resultcode == TWGManagedObjectConstants.CLIENT_SERVICE_START_SUCCEEDED) {
                String ipcpath = obj.getPreferredPath();
                if(ipcpath == null) {
                    return;
                }
                MSCSGetFullConfigCommand cfg = new MSCSGetFullConfigCommand(ipcpath,
                    MSCSGetFullConfigCommand.CLUSTER_ENUM_GROUP |
                    MSCSGetFullConfigCommand.CLUSTER_ENUM_NODE);
                cfg.setCommandCompleteListener(this);
                try {
                    TWGServerServiceNode.GetServiceNode().SendAsynchCommand(cfg);
                } catch (ServiceNodeException snx) {
                    TWGRas.error(TWGRas.ENGINE, "Error sending cluster config request");
                } catch (InterruptedException ix) {
                }
            }
        }
        /**
         * Notification method for completion of StopClientService() request
         * @param mo - TWGManagedObject which request was related to
         * @param resultcode - result code of the request
         */
        public void ClientServiceStopCompleted(TWGManagedObject mo,
            int resultcode) {
        }
    }
    /**
     * Address entry record support method.  This method generates a single
     * record encoding to be interpreted by MSCSAddressEntry.  The
     * record simply consists of the compressed unicode encoding of the
     * current best IPC path.
     * should be subclassed by MOF subclasses in order to provide support
     * for encoding address records.  Each address record must consist of
     * a list of records (1 or more), ordered from superclass to subclass,
     * with each record consisting of an overall record length (SHORT) and
     * a magic number (SHORT), followed by class-specific data.
     * @param obj - Managed object to request address encoding for
     * @return byte array containing encoded address record, or null no
     * support
     */
    protected byte[] getOurAddressEntryRecord(TWGManagedObject obj) {
        String path = ((MSCSManagedObject)obj).getPreferredPath();
        if(path == null)
            path = "";
        return MSCSAddressEntryShadow.buildAddressEntryRecord(path);
    }
    /**
     * Handler method for delivering TWGServerCommands for Native Managed
     * Object specific commands
     * @param cmdname - name of extension command
     * @param cmd - command instance received
     */
    public void ServerCommandReceived(String cmdname, Command cmd) {
        /* If request to create new cluster object                          */
        if(cmdname.equals(MSCSAddObjectCmd.CMDNAME)) {
            if(cmd.NumInputParms() < 3) {
                cmd.SetReturnCode(2);
                return;
            }
            /* Read input parameters                                	    */
            String nam = IntelByteBuffer.ReadCompUnicode(cmd.InputParm(1), 0);
            String cnam = IntelByteBuffer.ReadCompUnicode(cmd.InputParm(2), 0);
            if((nam.length() == 0) || (cnam.length() == 0)) {
                cmd.SetReturnCode(2);
                return;
            }
            /* If cluster already defined, report error			    */
            if(MSCSManagedObject.findMSCSObjectByName(cnam) != null) {
                cmd.SetReturnCode(3);
                return;
            }
            /* Create new object					    */
            try {
                MSCSManagedObject mo = new MSCSManagedObject(cnam, "",
                    new IntValueSet());
                mo.setName(nam);
                mo.save();
            } catch (TWGPersistentObjectSaveException posx) {
                TWGRas.error(TWGRas.ENGINE, "Error creating MSCS object");
                cmd.SetReturnCode(4);
                return;
            }
            cmd.SetReturnCode(0);
        }
    }
}

