package com.tivoli.twg.mscs;

import java.awt.Color;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.util.Locale;
import java.util.MissingResourceException;

import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JTextField;

import com.tivoli.twg.console.TWGAddMOFObjectDialog;
import com.tivoli.twg.console.TWGAddMOFObjectInterface;
import com.tivoli.twg.console.TWGConsoleColor;
import com.tivoli.twg.engine.TWGConsoleServiceNode;
import com.tivoli.twg.guilibs.TWGDialogPanel;
import com.tivoli.twg.libs.Command;
import com.tivoli.twg.libs.CommandCompleteListener;
import com.tivoli.twg.libs.MultiLocaleBundle;
import com.tivoli.twg.libs.ServiceNode;
import com.tivoli.twg.libs.ServiceNodeException;
import com.tivoli.uif.controls.UFButton;

/**
 * Create new Microsoft Cluster panel
 * @author Mike Primm
 * @version 1.1 - 6/8/05
 */
public class MSCSNewObjectPanel extends TWGDialogPanel 
implements TWGAddMOFObjectInterface,
KeyListener
{
    private JLabel sysLabel;
    private JTextField sysname;
    private JLabel clusLabel;
    private JTextField clusname;
    private MultiLocaleBundle mlb = null;

    private TWGAddMOFObjectDialog parentDialog = null;

    // Name of our NLS Bundle class                               
    private static final String OURNLSCLASS =
    "com.tivoli.twg.mscs.MSCSNLS";


    // Begin private inner class AddComplete
    /** Completion notification listener object 
     * for add command   
     */
    private static class AddComplete implements CommandCompleteListener,
    Runnable
    {
        private static final int GOOD = 0;
        private static final int DUPNAME = 1;
        private static final int BADOTHER = 2;
        int rslt;
        MultiLocaleBundle mlb;
        TWGAddMOFObjectDialog parent;
        AddComplete(TWGAddMOFObjectDialog dlg, MultiLocaleBundle nls) 
        {
            mlb = nls; 
            parent = dlg;
        }

        /**
         * Command complete notification
         * @param cmd - completed command
         */
        public void CommandComplete(Command cmd)
        {
            MSCSAddObjectCmd nmoc = (MSCSAddObjectCmd)cmd;
            // Check result of command				    
            if ( nmoc.wasSuccessful() )
            {
                rslt = GOOD;
            }
            else if ( nmoc.wasDuplicateClusterName() )
            {
                rslt = DUPNAME;
            }
            else
            {
                rslt = BADOTHER;
            }
            // If parent window found, display result dialog	    
            if ( parent != null )
            {
                Thread thrd = new Thread(this);
                thrd.start();
            }
            else
            {
                // If status indicator is running, parent will stop it
                // during cleanup.
                parent.dismissDialog();
            }
        }

        /**
         * Run method : displays result dialog box
         */
        public void run() 
        {
            // If good, inform user that object may not appear 
            // immediately                                    	    
            if ( rslt == GOOD )
            {
                String goodtitle = getString("addobject.goodrslt.title");
                String goodmsg = getString("addobject.goodrslt.msg");
                parent.setDismissDialog( true );
                parent.displayMessage( goodmsg, goodtitle, JOptionPane.INFORMATION_MESSAGE );
            }
            // If duplicate cluster name, display error             
            else if ( rslt == DUPNAME )
            {
                String dupnametitle = getString("addobject.dupnamerslt.title");
                String dupnamemsg = getString("addobject.dupnamerslt.msg");
                parent.displayMessage( dupnamemsg, dupnametitle, JOptionPane.WARNING_MESSAGE );
            }
            // Else, some other error occurred                      
            else
            {
                String badtitle = getString("addobject.badothrrslt.title");
                String badmsg = getString("addobject.badothrrslt.msg");
                parent.displayMessage( badmsg, badtitle, JOptionPane.ERROR_MESSAGE );
            }

            // stop the status indicator
            parent.stopStatusIndicator();
        }

        public String getString(String id)
        {
            String name = id;
            if ( mlb != null )
            {
                name = mlb.getString(Locale.getDefault(), id, id);
            }
            return name;
        }
    }
    // End private inner class AddComplete



    /**
     * Default constructor for new object panel
     */
    public MSCSNewObjectPanel()
    {
        // Load our NLS resources                                 
        try
        {
            mlb = MultiLocaleBundle.getSharedBundle(OURNLSCLASS);
        }
        catch ( MissingResourceException mrx )
        {
            mlb = null;
        }
        // Build disabled panel : we will enable when we get IPC  
        // protocol data                                          
        BuildView();
    }


    /**
     * Build panel view
     */
    private void BuildView()
    {
        GridBagLayout gbl = new GridBagLayout();
        GridBagConstraints gbc = new GridBagConstraints();

        setLayout(gbl);
        gbc.fill = GridBagConstraints.BOTH; // Fill both vertical+horiz 
        gbc.ipadx = 5;
        gbc.ipady = 1;
        gbc.insets = new Insets(5, 5, 5, 5);

        // Create fields for system name entry field                        
        sysLabel = new JLabel(getString("addpanel.label.sysname"));
        gbc.gridwidth=GridBagConstraints.RELATIVE;
        gbl.setConstraints(sysLabel, gbc);
        add(sysLabel);
        sysname = new JTextField("", 32);
        sysname.setEditable(true);
        sysname.addKeyListener(this);
        gbc.gridwidth=GridBagConstraints.REMAINDER;
        gbl.setConstraints(sysname, gbc);
        add(sysname);

        // Create fields for cluster name                                   
        clusLabel = new JLabel(getString("addpanel.label.clusname"));
        gbc.gridwidth=GridBagConstraints.RELATIVE;
        gbl.setConstraints(clusLabel, gbc);
        add(clusLabel);

        clusname = new JTextField("", 32);
        clusname.setEditable(true);
        clusname.addKeyListener(this);
        gbc.gridwidth=GridBagConstraints.REMAINDER;
        gbl.setConstraints(clusname, gbc);
        add(clusname);
    }


    /** 
     * Set the background color for this panel 
     */
    public void setBackground( Color c )
    {
        super.setBackground( c );
    }


    /** 
     * Set the foreground color for this panel 
     */
    public void setForeground( Color c )
    {
        super.setForeground( c );
        if ( sysLabel != null )
            sysLabel.setForeground( c );
        if ( clusLabel != null )
            clusLabel.setForeground( c );
    }


    /**
     * Load NLS string with given ID, or use ID if string not found
     * @param id - string ID
     * @return string loaded, or ID string if not found
     */
    public String getString(String id)
    {
        String name = id;
        if ( mlb != null )
        {
            name = mlb.getString(Locale.getDefault(), id, id);
        }
        return name;
    }


    // Begin Required for TWGAddMOFObjectInterface 
    public void setParentDialog( TWGAddMOFObjectDialog parent )
    {
        parentDialog = parent;
        okIsAllowed();
    }


    public boolean displayStatusBar()
    {
        return true;
    }


    public void setFocus()
    {
        sysname.requestFocus();
    }


    /**
     * Allows subclassing applications to verify information entered
     * before the dialog is dismissed and to prevent dismissal.
     *
     * @return Return true to allow the dialog to close.  Return false
     * to prevent the dialog from closing.
     */
    public boolean verifyOnClose()
    {
        // Read fields from panel				
        String sname = sysname.getText();
        String cname = clusname.getText();

        // Build and send AddObject command to engine           
        MSCSAddObjectCmd cmd = new MSCSAddObjectCmd( sname, cname );
        cmd.setCommandCompleteListener(new AddComplete(parentDialog, 
                                                       mlb));
        try
        {
            TWGConsoleServiceNode.getConsoleServiceNode().SendAsynchCommand(cmd);
            parentDialog.startStatusIndicator();
        }
        catch ( ServiceNodeException snx )
        {
            System.out.println("Error sending add object request");
        }
        catch ( InterruptedException ix )
        {
        }
        return true;
    }


    public void pCleanUp()
    {
        if ( sysname != null )
        {
            sysname.removeKeyListener( this );
            clusname.removeKeyListener( this );
            removeAll();
            sysname = null;
            clusname = null;
        }
    }
    // End Required for TWGAddMOFObjectInterface



    /**
     * Implement a key listener to update okay button
     * @see KeyListener
     */
    public void keyPressed(KeyEvent e)
    {
    }
    public void keyTyped(KeyEvent e)
    {
    }
    public void keyReleased( KeyEvent e )
    {
        okIsAllowed();
    }


    /**
     * Convenience method for checking the current contents of the entry fields.
     * @return Return true if both entry fields contain data. Otherwise
     *         return false if either of them are empty.
     */
    private boolean checkInputData()
    {
        boolean brc = false;
        if ( sysname != null )
        {
            String sname = sysname.getText();
            String cname = clusname.getText();
            brc = ( !( ( sname == null || sname.trim().equals( "" ) )
                       || ( cname == null || cname.trim().equals( "" ) )) );
        }
        return brc;
    }


    /**
     * Convenience method for enabling/disabling the OK button on the
     * dialog based on the current contents of the entry fields.
     */
    private void okIsAllowed()
    {
        // If we have access to our parent, and its a TWGAddMOFObjectDialog, then 
        // enable/disable the button based on the data                      
        if ( parentDialog != null )
        {
            boolean goodinput = checkInputData();
            parentDialog.getOKButton().setEnabled( goodinput );
            UFButton button = null;
            if ( goodinput )
            {
                button = parentDialog.getOKButton();
            }
            else
            {
                button = parentDialog.getCancelButton();
            }
            if ( button != null )
            {
                parentDialog.getRootPane().setDefaultButton(button);
            }
        }
    }
}
