/******************************************************************************
 * Licensed Materials - Property of IBM                                       *
 *                                                                            *
 * (C) Copyright by IBM 2003.  All Rights Reserved.                           *
 *****************************************************************************/

package com.ibm.sysmgt.sdk.tools;

import java.io.File;
import java.io.FilenameFilter;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.NotSerializableException;
import java.io.ObjectStreamException;

import java.net.InetAddress;
import java.net.UnknownHostException;

import java.util.Enumeration;
import java.util.NoSuchElementException;
import java.util.Observable;
import java.util.Observer;
import java.util.Properties;
import java.util.Timer;
import java.util.TimerTask;
import java.util.Vector;

import com.ibm.sysmgt.sdk.CmdTask.GetCmdStringCommand;
import com.ibm.sysmgt.sdk.CmdTask.CmdTaskData;

import com.tivoli.twg.engine.TWGTask;
import com.tivoli.twg.engine.TWGTaskActivation;
import com.tivoli.twg.engine.TWGTaskServer;

import com.tivoli.twg.libs.BadServiceNodeImplException;
import com.tivoli.twg.libs.Command;
import com.tivoli.twg.libs.CommandReceivedListener;
import com.tivoli.twg.libs.PropertiesFile;
import com.tivoli.twg.libs.ServiceNode;
import com.tivoli.twg.libs.ServiceNodeException;
import com.tivoli.twg.libs.ServiceNodeClosedException;
import com.tivoli.twg.libs.TWGEnvironment;
import com.tivoli.twg.log.TWGRas;


/**
 * This class implements TWGTaskServer to process UserDefinedManagedSystem
 * commands on the server.
 *
 * @see com.tivoli.twg.engine.TWGTaskServer
 * @see com.tivoli.twg.libs.CommandReceivedListener
 */
public class SDKCLITaskServer implements TWGTaskServer, CommandReceivedListener
{
  /* Constants */
  public static final int USER_DEFINED_SYSTEM_CREATION = 15;
  public static final int CMDTASK_GET_COMMANDSTRING = 16;
  public static final String TASK_SERVER_CLASS = "com.ibm.sysmgt.sdk.tools.SDKCLITaskServer";
  public static final int GOOD = 0;
  public static final int BADADDR = 1;
  public static final int BADOTHER = 2;
  public static final int DUPLICATE = 3;
  public  static final String SERVICE_NODE_NAME = "SDKCLITask";

  /* Private Variables */
  private static ServiceNode sn = null;
  private volatile boolean keepGoing = true;
  private boolean DoCallback;

  /**
   * Constructor
   */
  public SDKCLITaskServer()
  {
    TWGRas.entryExit(TWGRas.SDK, "SDKCLITaskServer constructor entry");
  }

  /**
   * This method is called prior to starting the thread with this object so
   * that this object can access its TWGTask object.  This method will be
   * called before a thread is started for this object (i.e. before run() is
   * called).
   *
   * @param task  the TWGTask object for this server task.
   */
  public boolean serverActivate(TWGTask task)
  {
    /* Start the service node for the task here */
    boolean rc = true;
    Exception exception = null;

    TWGRas.entryExit(TWGRas.SDK, "SDKCLITaskServer.serverActivate() entry");

    try
    {
      sn = new ServiceNode(SERVICE_NODE_NAME, false);
      sn.RegisterDefaultCommandListener(this);
      if (sn.Create() == false)
      {
        rc = false;
        exception = new ServiceNodeException();
      }
    }
    catch (BadServiceNodeImplException bsnie)
    {
      rc = false;
      exception = bsnie;
    }
    catch (ServiceNodeClosedException snce)
    {
      rc = false;
      exception = snce;
    }
    catch (ServiceNodeException sne)
    {
      rc = false;
      exception = sne;
    }

    if (rc == false)
    {
      exception.printStackTrace();
      TWGRas.error(TWGRas.SDK, "SDKCLITaskServer.serverActivate() ServiceNode exception"
                   + exception);
    }

    TWGRas.entryExit(TWGRas.SDK, "SDKCLITaskServer.serverActivate() exit, rc = " + rc);

    return rc;
  }

  /**
   * Called once the server is deactivating this task
   */
  public void serverDeactivate()
  {
    TWGRas.entryExit(TWGRas.SDK, "SDKCLITaskServer.serverDeactivate() entry");

    try
    {
      sn.DeregisterDefaultCommandListener();
      try
      {
        sn.Close();
      }
      catch (ServiceNodeException exception)
      {
      }
    }
    catch (NullPointerException exception)
    {
    }

    TWGRas.entryExit(TWGRas.SDK, "SDKCLITaskServer.serverDeactivate() exit");
  }

  /**
   * This method is called when a subtask activation has been received from
   * the console for this task.  At a bare minimum, the TWGTaskServer must
   * call act.subtaskActivated() with either TWGTaskConstants.RETURN_OK
   * or TWGTaskConstants.GENERAL_FAILURE to indicate success or failure for
   * activating the subtask.
   *
   * @param act  the TWGTaskActivation object for the subtask being activated.
   */
  public int subtaskActivate(TWGTaskActivation act)
  {
    return TWGTask.SUBTASK_ACT_COMPLETE;
  }

  /**
   * This method is called when a subtask deactivation has been received from
   * the console for this task.
   */
  public void subtaskDeactivate(TWGTaskActivation act)
  {
  }

  /**
   * This method is part of being runnable which the TWGTaskServer interface
   * extends.  For this test case, we'll simply sit in a loop until we're told
   * to exit by serverDeactivate().
   */
  public void run()
  {
    Thread ThisThread = Thread.currentThread();
    TWGRas.entryExit(TWGRas.SDK, "SDKCLITaskServer.run() entry");

    try
    {
      if (ThisThread != null)
      {
        ThisThread.setName("SDKCLITaskServer");
      }
    }
    catch (SecurityException exception)
    {
    }

    while (keepGoing)
    {
      try
      {
        TWGRas.debug(TWGRas.SDK, "SDKCLITaskServer.run(): calling ProcessCommands()");
        sn.ProcessCommands();
      }
      catch (ServiceNodeClosedException exception)
      {
        keepGoing = false;
        TWGRas.error(TWGRas.SDK, "SDKCLITaskServer.run(): ServiceNodeClosedException: " + exception);
      }
      catch (ServiceNodeException exception)
      {
        TWGRas.error(TWGRas.SDK, "SDKCLITaskServer.run(): ServiceNodeException: " + exception);
      }
      catch (InterruptedException exception)
      {
        TWGRas.error(TWGRas.SDK, "SDKCLITaskServer.run(): InterruptedException: " + exception);
      }
    }
  }

  public static ServiceNode getSDKCLITaskServiceNode() throws ServiceNodeClosedException
  {
    if (sn == null)
    {
      throw new ServiceNodeClosedException();
    }

    return sn;
  }

  public boolean CommandReceived(ServiceNode CallBackSN, Command cmd)
  {
    TWGRas.entryExit(TWGRas.SDK, "SDKCLITaskServer.CommandReceived() entry, cmd = " + cmd.CommandCode());

    /* What type of command is this here */
    switch ((int)cmd.CommandCode())
    {
      case USER_DEFINED_SYSTEM_CREATION:
        /* A request to create a new device from the Console based on user input */
        int retStatus = BADOTHER;

        /* Make sure we received 2 parameters */
        if (cmd.NumInputParms() == 2)
        {
          try
          {
            String hostName = (String)cmd.GetObjectFromInputParm(0);
            //String address = (String)cmd.GetObjectFromInputParm(1);

            /* Now try to add the device to the list */
            //if ((hostName != null) && (address != null))
            if (hostName != null)
            {
              /* Verify IP address for hostname */
              InetAddress destAddress = null;
              String ipAddress = null;
              try
              {
                destAddress = InetAddress.getByName(hostName);
                ipAddress = destAddress.getHostAddress();
                TWGRas.debug(TWGRas.SDK, "SDKCLITaskServer.CommandReceived(): destAddress = " +
                             destAddress + ", ipAddress = " + ipAddress);
              }
              catch (UnknownHostException exception)
              {
                TWGRas.error(TWGRas.SDK, "SDKCLITaskServer.CommandReceived(): Can't resolve IP address for host \"" +
                             hostName + "\"");
                retStatus = BADADDR;
              }
              if (ipAddress != null)
              {
                if (UserDefinedManagedSystem.getUserDefinedObject(hostName) == null)
                {
                  TWGRas.debug(TWGRas.SDK,
                               "SDKCLITaskServer.CommandReceived(): creating new UserDefinedManagedSystem: "
                               + hostName);
                  // create the new user defined managed object
                  UserDefinedManagedSystem newSystem = new UserDefinedManagedSystem(hostName);
                  // ping the managed object to set the state correctly.
                  if (newSystem == null)
                  {
                    retStatus = BADOTHER;
                  }
                  else
                  {
                    newSystem.startPing();
                    retStatus = GOOD;
                  }
                }
                else
                {
                  retStatus = DUPLICATE;
                }
              }
            }
          }
          catch (ObjectStreamException exception)
          {
          }
          catch (ClassNotFoundException exception)
          {
          }
        }
        try
        {
          cmd.AddObjectAsOutputParm(new Integer(retStatus));
          //cmd.AddObjectAsOutputParm(new Long(moid));
        }
        catch (NotSerializableException exception)
        {
        }
        break;

      default:
        break;
    }
    return true;
  }


}
