//------------------------------------------------------------------------------
// File Name   : SampleMonlet.java
// Description : Supplies system time information via the Monlet interface
//
// (c) Copyright IBM 1999. All rights reserved.
// Change Activity:
//   02-16-99 - Ron Harrod              Author
//   08-20-99 - J Pechacek              Add locale support
//------------------------------------------------------------------------------
package com.BobCo.monlet;

import com.tivoli.twg.agent.TWGAgentExtension;
import com.tivoli.twg.agent.TWGAgentExtensionException;
import com.tivoli.twg.libs.NotifyClock;
import com.tivoli.twg.libs.MultiLocaleBundle;
import com.tivoli.twg.log.TWGRas;
import com.tivoli.twg.monlet.MonitorConstants;
import com.tivoli.twg.monlet.Monlet;
import com.tivoli.twg.monlet.MonletException;
import com.tivoli.twg.monlet.MonletIndepCmd;
import com.tivoli.twg.monlet.MonletInvalidHandleException;
import com.tivoli.twg.monlet.MonletAttrMetaData;
import com.tivoli.twg.monlet.MonletAttributeNode;
import com.tivoli.twg.monlet.TWGMonPath;
import java.util.Date;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Locale;
import java.util.MissingResourceException;
import java.util.ResourceBundle;
import java.util.Vector;

/**
* A simple Monlet returning current time in various forms for testing
* @author  Ron Harrod
* @version 0.01, 06-28-99
*/
public class SampleMonlet extends TWGAgentExtension
  implements Runnable, Monlet, MonitorConstants {
  //----------------create the periodic data------------------------------------
  private Thread iTd;                           // this thread
  private Date iTick = new Date(System.currentTimeMillis()); // most recent tick
  private NotifyClock iSampleClock;             // supplies the interval
  private MultiLocaleBundle nlsResources;
  private String resourceBundleName;
  private String rootName;
  private static final String ROOTNODE = "SampleMonlet.TWGMonlet";

  /**
   * Override TWGAgentExtension method
   * Initialize this monlet from specified properties
   */
  public void initAgentExtension()
  throws TWGAgentExtensionException
  {
     /* Load NLS resource bundle */
     resourceBundleName = getProperty(TWGAgentExtension.EXT_NLS_CLASS);
     if (resourceBundleName != null) {
        try {
           nlsResources = new MultiLocaleBundle(resourceBundleName);
        } catch (MissingResourceException e) {
           TWGRas.error(TWGRas.MONITORS,"Resource bundle not found: " + nlsResources);
        }
     }
     /* Initialize root node name */
     String rootName = getProperty(MONLET_ROOT_NAME_PROPERTY).trim();
     if (rootName == null) {
        throw new TWGAgentExtensionException("Required monlet property not defined: " +
                                             MONLET_ROOT_NAME_PROPERTY);
     }
  }

  /**
   * Override TWGAgentExtension method
   * Handle extension termination
   */
  public void terminateAgentExtension()
  throws TWGAgentExtensionException
  {
     /* Perform any cleanup here */
  }

  /** Create and start a thread that updates on a specified interval */
  public SampleMonlet() {
    if (TWGRas.isDebugEnabled(TWGRas.MONITORS))
       TWGRas.entry(TWGRas.MONITORS,"Constructing SampleMonlet");
    iSampleClock = new NotifyClock(5000);       // refresh at each 5 second tick
    iTd = new Thread(this, "Time");             // make a thread to run
    iTd.setDaemon(true);                        // die with this instance
    iTd.start();                                // run() this thread
    if (TWGRas.isDebugEnabled(TWGRas.MONITORS))
       TWGRas.exit(TWGRas.MONITORS,"Exiting SampleMonlet constructor");
  }

  /** @see java.lang.Runnable#run() */
  public void run() {
    for (;;) {
      iTick = new Date(iSampleClock.nextTick()); // wait for next update time
      if (TWGRas.isDebugEnabled(TWGRas.MONITORS))
         TWGRas.debug(TWGRas.MONITORS,"SampleMonlet.run(): tick=" + getMs() +
                                                  " " + getSecs() + " " + this);
    }
  }

  /** @see java.lang.Thread#stop() */
  public void stop() { iTd.stop(); }

  //----------------create some simple nodes for data presentation--------------
  private static final Hashtable NODES = new Hashtable(3);
  private static final String MILLISECS = "TimeMilliseconds";
  private static final String SECONDS   = "TimeSeconds"     ;
  private static final String FORMATTED = "TimeFormatted"   ;
  static {
    NODES.put(MILLISECS, "%10.0lf");
    NODES.put(SECONDS  , "%10.0lf");
    NODES.put(FORMATTED, "%s"     );
  }

  //----------------implement the Monlet interface------------------------------
  /** @see com.tivoli.twg.monlet.Monlet */
  private Hashtable iHandles = new Hashtable();  // handle gets path

  public MonletAttributeNode[] MonletGetNodes(Vector aPath, Locale aLocale)
                                                    throws MonletException {
    if (TWGRas.isDebugEnabled(TWGRas.MONITORS))
       TWGRas.entry(TWGRas.MONITORS,
                  "Monlet " + rootName + " get nodes for path " + aPath +
                    " locale " + aLocale.toString());
    if (aPath.size() != 1  ||
                     !aPath.elementAt(0).equals(ROOTNODE))
                     throw new MonletException("Path not supported: " + aPath);
    MonletAttributeNode[] aResult = new MonletAttributeNode[NODES.size()];
    String aNode, aName;
    int i = 0; // satisfy compiler
    Enumeration aKeys = NODES.keys();
    while (aKeys.hasMoreElements()) {
      aNode = (String)aKeys.nextElement();
      aName = getNLSName(aNode, aLocale);
      aResult[i++] =new MonletAttributeNode(aName, // display (NLS) name
                                            aNode, // programmatic name
                                            aLocale,
                                            MonletAttributeNode.ATTRIBUTE_TYPE,
                                            getType(aNode));
      if (TWGRas.isDebugEnabled(TWGRas.MONITORS))
         TWGRas.debugH(TWGRas.MONITORS,"SampleMonlet.MonletGetNodes: " +
                                             " element("+i+"): " + aResult[i-1]);
    } // endwhile
    if (TWGRas.isDebugEnabled(TWGRas.MONITORS))
       TWGRas.exitH(TWGRas.MONITORS,
                  "Exiting SampleMonlet.MonletGetNodes: get tree information");
    return aResult;
  }

  private String getNLSName(String name, Locale locale)
  {
     return (nlsResources != null) ?
        nlsResources.getString(locale, name, name) :
        name;
  }

  public Object MonletGetData(int handle) throws MonletException {
    if (TWGRas.isDebugEnabled(TWGRas.MONITORS))
       TWGRas.entryH(TWGRas.MONITORS,
        "Entering SampleMonlet.MonletGetData: get data for handle: " + handle);
    String aPath = getPath(handle);
    Object aResult = null;                       // RC_INVALID_DATA return code
    if (TWGRas.isDebugEnabled(TWGRas.MONITORS))
       TWGRas.debugH(TWGRas.MONITORS,
                                  "SampleMonlet.MonletGetData: Path=" + aPath);
    if (aPath == null) {
      throw new MonletException();
    }
    try {
      if      (aPath.equals(MILLISECS)) aResult = new Long(getMs());
      else if (aPath.equals(SECONDS)) aResult = new Long(getSecs());
      else if (aPath.equals(FORMATTED)) aResult = getString();
      else TWGRas.error(TWGRas.MONITORS, "SampleMonlet.MonletGetData:" +
                           "Monlet data access failed to match path: " + aPath);
    } // endtry
    catch (NullPointerException e) {
      TWGRas.error(TWGRas.MONITORS, "SampleMonlet.MonletGetData:" +
         "Monlet data access failure for: " + aPath + "\nData: " + aResult , e);
      throw new MonletException();
    } // endcatch
    if (TWGRas.isDebugEnabled(TWGRas.MONITORS))
       TWGRas.exitH(TWGRas.MONITORS,
          "Exiting SampleMonlet.MonletGetData: get data for handle: " + handle);
    return aResult;
  }

  // Note: throws MonletException not used
  public MonletAttrMetaData MonletGetMetaData(int handle)
                                           throws MonletInvalidHandleException {
    String aPath = getPath(handle);
    if (aPath == null)
      throw new MonletInvalidHandleException("Invalid handle: "+  handle);
    MonletAttrMetaData aResult = new MonletAttrMetaData();
    aResult.setFormatStr(getFormat(aPath));
    aResult.setUnitsLabel(getFormat(aPath));
    aResult.setDataType(getType(aPath));
    return aResult;
  }

  public void MonletAssignHandle(Vector aPath, int handle)
             throws MonletException {
    if (null == NODES.get(getPath(aPath)))
                           throw new MonletException("Invalid path: " + aPath);
    // note: normally you must check for attribute type too
    iHandles.put(new Integer(handle), aPath);
    return;
  }

  public void MonletReleaseHandle(int handle) throws MonletException {
    if (null == iHandles.remove(new Integer(handle)))
       throw new MonletException("Unknown handle: " + handle);
    return;
  }

  public void GetDefaultThresholdPaths(Vector progNames,  /* input / output */
                                       Vector dispNames,  /* input / output */
                                       Locale aLocale)    {

    Vector pPath = new Vector(), dPath = new Vector();

    pPath.addElement(ROOTNODE); // programmatic names path
    pPath.addElement(SECONDS);

    dPath.addElement(getNLSName(ROOTNODE, aLocale)); // display names path
    dPath.addElement(getNLSName(SECONDS, aLocale));

    progNames.addElement(pPath);                 // fill in return Vectors
    dispNames.addElement(dPath);
  }

  public MonletIndepCmd MonletIndepCmd(TWGMonPath path,          /* input */
                                       MonletIndepCmd iCmd)   /* input */
  {
     return new MonletIndepCmd();
  }

  //----------------helper routines-------------------------------------------
  public long getMs() { return iTick.getTime(); }
  public long getSecs() { return getMs()/1000; }
  // protect from String becoming the four character String "null"
  public String getString() { return iTick == null ? null : ""+this; }

  private String getPath(Vector aPath) { return ""+aPath.lastElement(); }

  private String getPath(int handle) {
    return getPath((Vector)iHandles.get(new Integer(handle)));
  }

  private String getFormat(String aPath) { return (String)NODES.get(aPath); }

  private byte getType(String aPath) {
    return (getFormat(aPath).equals("%s") ?
                                   (byte)STRING_NODEDATA :
                                   (byte)DOUBLE_NODEDATA);
  }

  public String toString() { return ""+iTick; }
}

























