/******************************************************************************
 * Licensed Materials - Property of IBM                            *
 *                                                                            *
 * PID (5697-TKT)                                                             *
 *                                                                            *
 * (C) Copyright by IBM 1999. All Rights Reserved. *
 *****************************************************************************/
package com.BobCo.communication;

import com.tivoli.twg.libs.Command;
import com.tivoli.twg.libs.CommandCompleteListener;
import com.tivoli.twg.libs.IntelByteBuffer;
import com.tivoli.twg.libs.ServiceNode;
import com.tivoli.twg.libs.ServiceNodeClosedException;
import com.tivoli.twg.libs.ServiceNodeException;
import com.tivoli.twg.libs.ServiceNodeImplFactory;
import com.tivoli.twg.libs.ServiceNodeLocalImplFactory;

/**
 * SrvTask demonstrates how a server task can communicate with an agent
 * program.  SrvTask sends a message synchronously to the msglogr program,
 * waits for the return code and prints a completion message.
 */
public class SrvTask {

   // Service node instance.
   private ServiceNode svcNode;
   // Command complete received flag.
   private boolean cmdCompleteReceived = false;
   // Destination address.
   private String dest;

   /**
    * Simple constructor for the SrvTask class.
    */
   SrvTask() {}

   /**
    * Main method. Creates a service node.  Creates a message.  Sends the
    * message and waits for command complete.  Closes service node after
    * command complete is received.
    */
   public static void main(String[] args) {

      // Create an instance of this class.
      SrvTask srvTask = new SrvTask();

      // If no parameter is given, msglogr must be running on the same system
      // that SrvTask is running on.
      if (args.length == 0) {
         srvTask.dest = "MessageB";
      }
      // If one parameter is given, msglogr test for a valid destination address
      // use it.
      else if (args.length == 1) {
         // Test for valid-looking destination address.
         if (args[0].startsWith("TCPIP::") || args[0].startsWith("NETBIOS::")) {
            srvTask.dest = args[0]+"::MessageB";
         }
         else {
            printUsage();
            System.exit(0);
         }
      }
      // All other parameters...
      else {
         printUsage();
         System.exit(0);
      }

      // Create the service node.
      try {
         // Service node will be created from local impl factory.
         Class cls = Class.forName( "com.tivoli.twg.libs.ServiceNodeLocalImplFactory");
         try {
            // Create a new instance of the service node impl factory class.
            ServiceNodeImplFactory fct = (ServiceNodeImplFactory) cls.newInstance();
            // Set the service node factory instance in the service node class.
            ServiceNode.SetServiceNodeFactory(fct);

            // Create a new service node
            srvTask.svcNode = new ServiceNode("ServerTask");
            // Test if service node has been created.
            if (srvTask.svcNode.Create())
               System.out.println("ServiceNode created");
            else {
               System.out.println("ServiceNode create failed");
               return;
            }
         }

         // Catch exceptions from instantiating impl factory, and creating
         // service node.
         catch (InstantiationException e1) {
            System.out.println("Instantiation exception");
         }
         catch (IllegalAccessException e2) {
            System.out.println("Illegal access error");
         }
         catch (ServiceNodeException e3) {
            System.out.println("Service Node error");
         }
      }
      // Catch exception from setting local impl factory class.
      catch (ClassNotFoundException e1) {
         System.out.println("Class load error -- not found.");
      }

      // Create message severity and text.
      int severity = 7;
      String msg = "Hello World.";

      // Convert severity and text to Intel format.
      byte[] severityBB = new byte[IntelByteBuffer.SIZEOF_LONG];
      IntelByteBuffer.WriteLONG(severityBB, severity, 0);

      byte[] msgBB = new byte[IntelByteBuffer.GetUTF8Length(msg)];
      IntelByteBuffer.WriteUTF8(msgBB, msg, 0);

      // Create a command with LogMsg command code.
      LogMsgCommand logCmd = new LogMsgCommand();

      // Add message severity and text to the command.
      logCmd.AddInputParm(severityBB);
      logCmd.AddInputParm(msgBB);

      // Set the message destination.
      logCmd.SetDestinationAddress(srvTask.dest);

      // Send the command synchronously.  Command complete is handled by
      // the LogMsgCommand class.
      try {
         if (!srvTask.svcNode.SendCommand(logCmd)) {
            System.out.println("SendCommand failed.");
         }
      }
      // Catch send command exceptions.
      catch (ServiceNodeClosedException e1) {
         System.out.println("Service node closed exception on SendCommand.");
      }
      catch (InterruptedException e2) {
         System.out.println("Interrupted exception on SendCommand.");
      }
      catch (ServiceNodeException e3) {
         System.out.println("Service node closed exception on SendCommand.");
      }

      // Close service node.
      System.out.println("Closing service node.");
      try {
         srvTask.svcNode.Close();
      }
      // Catch service node close exceptions.
      catch (ServiceNodeException e1) {
         System.out.println("Service node exception on Close.");
      }
   }

   // Print out the 'help' for this program.
   private static void printUsage() {
      System.out.println(
         "Usage:  TWGJava com.BobCo.communication.SrvTask [protocol::address]\n"+
         "        Where protocol is either TCPIP or NETBIOS; and address\n"+
         "        (appropriate address form for the protocol) is the address\n"+
         "        of the destination system where the msglogr program must be\n"+
         "        running.  (Note:  TCPIP addresses must be in dotted decimal\n"+
         "        format, host name resolution is not supported.) \n"+
         "        The msglogr program must be started before SrvTask is run.");
   }
}
