package com.ibm.sysmgt.sdk.tools;

import com.tivoli.twg.engine.TWGFilter;
import com.tivoli.twg.engine.TWGObject;
import com.tivoli.twg.engine.InvalidObjectIDException;
import com.tivoli.twg.engine.TWGPersistentObjectSaveException;
import com.tivoli.twg.engine.TWGPrivToken;
import com.tivoli.twg.engine.TWGTask;
import com.tivoli.twg.engine.TWGUser;
import com.tivoli.twg.engine.TWGUserConstants;
import com.tivoli.twg.engine.cli.ICliBundle;
import com.tivoli.twg.libs.DataValue;
import com.tivoli.twg.libs.LongValueSet;
import com.tivoli.twg.libs.TWGString;
import com.tivoli.twg.libs.TWGStringArray;
import com.tivoli.twg.libs.TWGInteger;
import com.tivoli.twg.libs.TWGLongArray;
import com.tivoli.twg.libs.StringValueSet;
import com.tivoli.twg.libs.UserIDSecurity;

import java.io.Writer;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.ListIterator;
import java.util.Locale;
import java.util.HashMap;
import java.util.MissingResourceException;
import java.util.ResourceBundle;
import java.util.StringTokenizer;

/**
 * Director User access CLI Bundle<p>
 * This CLI bundle implementation provides access to Director
 * Console Users and privileges<p>
 * <li>Help - provies event help
 * <li>List - will print a list of all user commands
 * <li>ListUsers [-r|-report|-t|-terse]
 * <li>ListUserAttributes [-r|-report|-t|-terse]
 * <li>ModifyUserAttributes <User> <Attrib1=Value1> ... <AttribN=ValueN>
 * <li>ListPrivilegeTokens [-r|-report|-t|-terse]
 * <li>ListTasks [-r|-report|-t|-terse]
 * <li>ListGroups [-r|-report|-t|-terse]
 * <li>AddGroupAccess <User> [-id|-name] <Group1> ... <GroupN>
 * <li>RemoveGroupAccess <User> [-id|-name] <Group1> ... <GroupN>
 * <li>AddTaskAccess <User> [-id|-name] <Task1> ... <TaskN>
 * <li>RemoveTaskAccess <User> [-id|-name] <Task1> ... <TaskN>
 * 
 * @see ICliBundle for the bundle interface
 *
 * @author Paul Vezzetti
 *  * @version 4.2 - 11/2/2003
 */
public class UserCli implements ICliBundle
{
    private static final String LanguageBundle = "com.ibm.sysmgt.sdk.tools.UserCliNLS";
    public static final String UserAttribPrefix = "User.";
    public static final String ATTRIB_EMAIL = TWGUserConstants.ATTRIB_ID_EMAIL;
    public static final String ATTRIB_PAGER = TWGUserConstants.ATTRIB_ID_PAGER;
    public static final String ATTRIB_PRIVSET = TWGUserConstants.ATTRIB_ID_PRIVSET;
    public static final String ATTRIB_GROUP_FLAG = "User.group_access_limited";
    public static final String ATTRIB_TASK_FLAG = "User.task_access_limited";
    public static final String ATTRIB_READ_ONLY_FLAG = "User.group_access_readonly";
    public static final String ATTRIB_TASK_ACCESS_LIST = TWGUserConstants.ATTRIB_ID_TASKACCESS;
    public static final String ATTRIB_FILTER_ACCESS_LIST = TWGUserConstants.ATTRIB_ID_FILTERACCESS;

    private ResourceBundle nlsBundle = null;

    private int reportingType;
    private static final int TERSE_LISTING = 1;
    private static final String TERSE_SWITCH = "-t";
    private static final int REPORT_LISTING = 2;
    private static final String REPORT_SWITCH = "-r";
    private static final int STANDARD_LISTING = 3;
    private static final String NAME_SWITCH = "-name";

    // List of user modifible attributes and corresponding TWGUser attribute
    private static final String [] userAttributesList = {
        UserCli.ATTRIB_EMAIL,                // email
        UserCli.ATTRIB_PAGER,                // pager info
        UserCli.ATTRIB_PRIVSET,              // privilege set
        UserCli.ATTRIB_GROUP_FLAG,           // group access flag
        UserCli.ATTRIB_READ_ONLY_FLAG,       // group read-only access
        UserCli.ATTRIB_TASK_FLAG,            // task access flag
        UserCli.ATTRIB_TASK_ACCESS_LIST,     // task access list
        UserCli.ATTRIB_FILTER_ACCESS_LIST    // filter access list
    };

    // List of supported user attributes information
    // This table is used in the ListUserAttributesCommand
    private final static String[][] userAttributeTypes = {
        {"ATTRIB.TYPE.STR",  "ATTRIB.DESC.EMAIL"}, // ATTRIB_ID_EMAIL
        {"ATTRIB.TYPE.STR",  "ATTRIB.DESC.PAGER"}, // ATTRIB_ID_PAGER
        {"ATTRIB.TYPE.STR_ARRAY",  "ATTRIB.DESC.PRIVS"}, // ATTRIB_ID_PRIVSET
        {"ATTRIB.TYPE.BOOL", "ATTRIB.DESC.BOOL"}, // ATTRIB_ID_ACCESSFLAGS
        {"ATTRIB.TYPE.BOOL", "ATTRIB.DESC.BOOL"}, // ATTRIB_ID_ACCESSFLAGS
        {"ATTRIB.TYPE.BOOL", "ATTRIB.DESC.BOOL"}, // ATTRIB_ID_ACCESSFLAGS
        {"ATTRIB.TYPE.LONG_ARRAY", "ATTRIB.DESC.TASKACCESS"}, // ATTRIB_ID_TASKACCESS
        {"ATTRIB.TYPE.LONG_ARRAY", "ATTRIB.DESC.FILTERACCESS"} // ATTRIB_ID_FILTERACCESS
    };

    /**
     * Bundle help command which must be implemented by all bundles
     * @param Locale locale - client invocation locale
     * @param String[] args - Command input data
     * @param Writer out - stream to client's stdout
     * @return int - client exit code
     */
    public int help(Locale locale, String[] args, Writer out) throws Exception
    {
        if ( nlsBundle == null )
            nlsBundle = getBundle(LanguageBundle, locale);
        out.write( nlsBundle.getString("TITLE") + "\r\n");
        return( list(locale, args, out) );
    }

    /**
     * CLI List command
     * Argument Input Format: no arguments expected
     */
    public int list(Locale locale, String[] args, Writer out) throws Exception
    {
        if ( nlsBundle == null )
            nlsBundle = getBundle(LanguageBundle, locale);

        try
        {
            int idx = 1;
            String str = nlsBundle.getString( "CLI_COMMANDS." +  idx );
            while ( str != null )
            {
                out.write( str + "\r\n" );
                str = nlsBundle.getString( "CLI_COMMANDS." +  ++idx );
            }
        }
        catch ( java.util.MissingResourceException e )
        {
        }
        return ICliBundle.OK;
    }

    /**
     * Cli command to retrieve the list of Director user information
     * Argument Input Format: [-r|-report|-t|-terse]
     */
    public int listUsers(Locale locale, String args[], Writer out) throws Exception
    {
        int i;
        setReportingLevel(args);

        if ( nlsBundle == null )
            nlsBundle = getBundle(LanguageBundle, locale);

        // Get all of native user ids (ie members of DirAdmin, DirSuper and Administrator )
        String [] nativeUserIDs = UserIDSecurity.getNativeUserIDs();
        // Get all of the TWGUser objects (ie initialized user ids)
        TWGUser [] allTWGUsers = TWGUser.getAllUsers();

        // Get all TWGUserID strings
        String [] twgUserIDs = new String[allTWGUsers.length];
        for ( i = 0; i < allTWGUsers.length; i++ )
        {
            twgUserIDs[i] = allTWGUsers[i].getName();
        }

        // Sort arrays before doing difference
        StringValueSet.Sort(nativeUserIDs, 0, nativeUserIDs.length);
        StringValueSet.Sort(twgUserIDs, 0, twgUserIDs.length);
        // Get list of ids which only exist in native OS
        String [] nativeOnlyUsers = StringValueSet.DifferenceOfArrays(nativeUserIDs, nativeUserIDs.length,
                                                                      twgUserIDs, twgUserIDs.length);
        // Terse Listing: Just list the user id's
        if ( reportingType == TERSE_LISTING )
        {
            out.write("\r\n");
            for ( i = 0; i < twgUserIDs.length; i++ )
            {
                out.write(twgUserIDs[i] + "\r\n");
            }
            for ( i = 0; i < nativeOnlyUsers.length; i++ )
            {
                out.write(nativeOnlyUsers[i] + "\r\n");
            }
            out.write("\r\n");
        }
        else if ( reportingType == STANDARD_LISTING )
        {                                             // Standard Listing: List modifible attributes.  Could be input to ModifyUserAttributes
            for ( i = 0; i < allTWGUsers.length; i++ )
            {
                TWGUser usr = allTWGUsers[i];

                out.write("\r\n" + usr.getName() + " ");
                if ( usr.getEmail().length() > 0 )
                    out.write( ATTRIB_EMAIL.substring(UserAttribPrefix.length()) + "=" + usr.getEmail() + " ");
                if ( usr.getPager().length() > 0 )
                    out.write( ATTRIB_PAGER.substring(UserAttribPrefix.length()) + "=" + usr.getPager() + " ");
                // Print the privilege strings
                TWGStringArray privs = (TWGStringArray)usr.getAttributeValue(TWGUserConstants.ATTRIB_ID_PRIVSET, locale);
                if ( (privs != null) && ( privs.getLength() > 0 ) )
                {
                    out.write(ATTRIB_PRIVSET.substring(UserAttribPrefix.length()) + "={" );
                    for ( int j = 0; j < privs.getLength(); j++ )
                    {
                        out.write(privs.getValueAt(j) );
                        if ( j != (privs.getLength() - 1) )
                            out.write(", ");
                    }
                    out.write("} ");
                }
                // Print the access flag values
                out.write(ATTRIB_GROUP_FLAG.substring(UserAttribPrefix.length()) + "=" + String.valueOf(usr.isFilterLimited()) + " ");
                out.write(ATTRIB_TASK_FLAG.substring(UserAttribPrefix.length()) + "=" + String.valueOf(usr.isTaskLimited()) + " ");
                out.write(ATTRIB_READ_ONLY_FLAG.substring(UserAttribPrefix.length()) + "=" + String.valueOf(usr.isReadOnlyFilterAccess()) + " ");
                // Print the group membership.  If group are not limited, nothing is printed.
                LongValueSet filters = usr.getFilterAccessList();
                if ( usr.isFilterLimited() && (filters != null) && (filters.Length() > 0 ) )
                {
                    out.write(ATTRIB_FILTER_ACCESS_LIST.substring(UserAttribPrefix.length()) + "={" );
                    for ( int j = 0; j < filters.Length(); j++ )
                    {
                        out.write(Long.toHexString(filters.GetValue(j)).toUpperCase() );
                        if ( j != filters.Length() - 1 )
                            out.write(", ");
                    }
                    out.write("} ");
                }

                LongValueSet tasks = usr.getTaskAccessList();
                if ( usr.isTaskLimited() && ( tasks != null ) && ( tasks.Length() > 0 ) )
                {
                    out.write(ATTRIB_TASK_ACCESS_LIST.substring(UserAttribPrefix.length())  + "={" );
                    for ( int j=0; j < tasks.Length(); j++ )
                    {
                        out.write( Long.toHexString(tasks.GetValue(j)).toUpperCase()  );
                        if ( j != tasks.Length() - 1 )
                            out.write(", ");
                    }
                    out.write("} ");
                }
                out.write("\r\n");

            }
            out.write("\r\n");
            for ( i = 0; i < nativeOnlyUsers.length; i++ )
            {
                out.write(nativeOnlyUsers[i]  + "\r\n");
            }

        }
        else if ( reportingType == REPORT_LISTING )
        {
            for ( i = 0; i < allTWGUsers.length; i++ )
            {
                TWGUser usr = allTWGUsers[i];

                out.write("\r\n" + getString(nlsBundle, "LIST.USER.NAME") + usr.getName() + "\r\n" );
                out.write(getString(nlsBundle, "LIST.USER.SUPERUSER") + UserIDSecurity.isSuperUser(usr.getName()) + "\r\n");
                out.write(getString(nlsBundle, "LIST.USER.FULLNAME") + usr.getFullName() + "\r\n" ); 
                out.write(getString(nlsBundle, "LIST.USER.DESC")  + usr.getDescription() + "\r\n" ); 
                out.write(getString(nlsBundle, "LIST.USER.EMAIL") + usr.getEmail() + "\r\n" ); 
                out.write(getString(nlsBundle, "LIST.USER.PAGER") + usr.getPager() + "\r\n" );
                out.write(getString(nlsBundle, "LIST.USER.LOCALE") + usr.getLocale() + "\r\n" );

                // Add groups
                out.write( formatGroupsListString(usr, locale) );

                // Add tasks
                out.write( formatTasksListString(usr, locale) );

                // Get all of the user's privileges
                TWGStringArray privs = (TWGStringArray)usr.getAttributeValue(TWGUserConstants.ATTRIB_ID_PRIVSET, locale);

                if ( (privs != null) && ( privs.getLength() > 0 ) )
                {
                    out.write(getString(nlsBundle,"LIST.USER.PRIVS")  + "{ ");
                    for ( int j = 0; j < privs.getLength(); j++ )
                    {
                        TWGPrivToken token = TWGPrivToken.findPrivToken(privs.getValueAt(j));
                        if ( token != null )
                        {
                            out.write(token.getName(locale) );
                            if ( j != privs.getLength() - 1 )
                            {
                                out.write(", ");
                            }
                        }
                    }
                    out.write(" }\r\n");
                }
            }
            // For native users, just show the available native information
            for ( i = 0; i < nativeOnlyUsers.length; i++ )
            {
                out.write("\r\n" + getString(nlsBundle, "LIST.USER.NAME") + nativeOnlyUsers[i] + "\r\n" );
                out.write(getString(nlsBundle, "LIST.USER.SUPERUSER") + UserIDSecurity.isSuperUser(nativeOnlyUsers[i]) + "\r\n");
                out.write(getString(nlsBundle, "LIST.USER.FULLNAME") + UserIDSecurity.getUserFullName(nativeOnlyUsers[i]) + "\r\n" ); 
                out.write(getString(nlsBundle, "LIST.USER.DESC")  + UserIDSecurity.getUserDescription(nativeOnlyUsers[i]) + "\r\n" ); 
            }
        }
        return ICliBundle.OK;
    }
    /**
     * Cli command to retrieve the list of user modifiable attributes.
     * These attributes are legal for use with modifyUserAttributes command
     * Argument Input Format: [-r|-report|-t|-terse]
     */
    public int listUserAttributes(Locale locale, String args[], Writer out) throws Exception
    {
        if ( nlsBundle == null )
            nlsBundle = getBundle(LanguageBundle, locale);

        setReportingLevel(args);

        for ( int i = 0; i < userAttributesList.length; i++ )
        {
            String name = userAttributesList[i].substring(UserAttribPrefix.length());
            String type = userAttributeTypes[i][0];
            String value = userAttributeTypes[i][1];
            if ( reportingType == STANDARD_LISTING )
            {
                out.write(nlsBundle.getString(type) + "::" + name + "\n");
            }
            else if ( reportingType == TERSE_LISTING )
            {
                out.write(name + "\n");
            }
            else
            {    /*   report listing   */
                out.write(nlsBundle.getString("LIST.ATTRIB.NAME") + name + "\n" );
                out.write(nlsBundle.getString("LIST.ATTRIB.TYPE") + nlsBundle.getString(type) + "\n");
                // Do not show translated boolean value keywords
                if ( !value.equalsIgnoreCase("ATTRIB.DESC.BOOL") )
                {
                    out.write(nlsBundle.getString("LIST.ATTRIB.VALUE") +  nlsBundle.getString(value) + "\n\n");
                }
                else
                {
                    out.write(nlsBundle.getString("LIST.ATTRIB.VALUE") + "True, False" + "\n\n");
                }
            }
        }

        return ICliBundle.OK;

    }
    /**
     * Cli command to modify any of the user modifiable attributes.
     * These attributes are legal for use with modifyUserAttributes command
     * Argument Input Format: user attrib(1)=value(1) attrib(2)={val2a,val2b} ... attrib(N)=Value(N) 
     */
    public int modifyUserAttributes(Locale locale, String args[], Writer out) throws Exception
    {
        if ( nlsBundle == null )
            nlsBundle = getBundle(LanguageBundle, locale);

        if ( args.length < 2 )
        {
            out.write(getString(nlsBundle, "ERROR.SYNTAX"));
            out.write(getString(nlsBundle, "CLI_COMMANDS.5"));
            return ICliBundle.USAGE;
        }
        // Try to get TWGUser object.  First arg is user id
        TWGUser user = findTWGUser( args[0] );

        if (user == null)
        {
            out.write(getString(nlsBundle, "ERROR.USER_NOT_FOUND"));
            return ICliBundle.USAGE;
        }

        // Check if user is a SuperUser.  SuperUsers can't be modified.
        if ( UserIDSecurity.isSuperUser(user.getName()) )
        {
            out.write(getString(nlsBundle, "ERROR.SUPERUSER"));
            return ICliBundle.USAGE;
        }

        // Parse the attribute arguments
        AttributeValuePair [] avp = parseAttributeArguments( args, out );
        // If no valid attributes, display error msg
        if ( avp.length == 0 )
        {
            out.write(getString(nlsBundle, "ERROR.ATTRIBS"));
            return ICliBundle.USAGE;
        }

        // Set each attribute on the user
        for ( int i = 0; i < avp.length; i++ )
        {
            out.write(avp[i].getAttribute() + ":\t" + avp[i].getDataValue().toNLSString(locale) + "\r\n");
            if ( !setAttributeValue(user, locale, avp[i].getAttribute(), avp[i].getDataValue()) )
                out.write(getString(nlsBundle,"ERROR.SET_ATTRIB_FAILED") + ": " + avp[i].getAttribute() + "\r\n");
        }
        // If we changed any values than we need to save the TWGUser object
        try
        {
            if ( user.isSaveRequired() )
                user.save();
        }
        catch ( TWGPersistentObjectSaveException pos )
        {
            out.write(getString(nlsBundle, "ERROR.SAVE_FAILED") + "\r\n");
            return ICliBundle.COMMAND_EXCEPTION;
        }

        return ICliBundle.OK;

    }
    /**
     * Cli command to list all of the available privilege tokens.  This command is offered
     * to display the available privilege strings which could be used with modifyUserAttributes.
     * Argument Input Format: [-r|-report|-t|-terse]
     */
    public int listPrivilegeTokens(Locale locale, String args[], Writer out) throws Exception
    {
        if ( nlsBundle == null )
            nlsBundle = getBundle(LanguageBundle, locale);

        setReportingLevel(args);
        // Get all of the privilege token id strings
        StringValueSet privs = TWGPrivToken.getTokenIDs();

        if ( reportingType == TERSE_LISTING )
        {
            for ( int i = 0; i < privs.Length(); i++ )
            {
                out.write(privs.GetValue(i) + "\r\n");
            }
        }
        else if ( reportingType == STANDARD_LISTING )
        {
            for ( int i = 0; i < privs.Length(); i++ )
            {
                TWGPrivToken token = TWGPrivToken.findPrivToken( privs.GetValue(i) );
                if ( token != null )
                {
                    String tokenName = token.getName(locale);
                    out.write(privs.GetValue(i) + "\t" + tokenName+ "\r\n");

                }
            }
        }
        else if ( reportingType == REPORT_LISTING )
        {
            for ( int i = 0; i < privs.Length(); i++ )
            {
                TWGPrivToken token = TWGPrivToken.findPrivToken( privs.GetValue(i) );
                if ( token != null )
                {
                    out.write(getString(nlsBundle,"LIST.PRIVTOKEN.ID") +  privs.GetValue(i) + "\r\n");
                    String tokenName = token.getName(locale);
                    out.write(getString(nlsBundle,"LIST.PRIVTOKEN.NAME") + tokenName+ "\r\n\n");

                }
            }

        }

        return ICliBundle.OK;

    }
    /**
     * Cli command to list all of the available user tasks.  This command is offered
     * to display the available task ids which could be used with modifyUserAttributes.
     * Argument Input Format: [-r|-report|-t|-terse]
     */
    public int listTasks(Locale locale, String [] args, Writer out) throws Exception
    {
        if ( nlsBundle == null )
            nlsBundle = getBundle(LanguageBundle, locale);

        setReportingLevel(args);

        try
        {   // Get all of the TWGTask objects
            Enumeration objs = TWGObject.AllObjects( Class.forName("com.tivoli.twg.engine.TWGTask") );

            while ( objs.hasMoreElements() )
            {
                Object obj = objs.nextElement();
                if ( (obj != null) && (obj instanceof TWGTask) )
                {
                    TWGTask task = (TWGTask)obj;
                    // We only show unrestricted, enabled, parent tasks.  This matches TWGUserTaskAccessPanel.java.
                    if ( (task != null) && (!task.isUnrestricted()) && (!task.isDisabled()) && (task.getParent() == null) )
                    {
                        if ( reportingType == TERSE_LISTING )
                        {
                            out.write( Long.toHexString(task.ObjectID()).toUpperCase() + "\r\n");
                        }
                        else if ( reportingType == STANDARD_LISTING )
                        {
                            out.write( Long.toHexString(task.ObjectID()).toUpperCase() + "\t" +  task.getTitle(locale) + "\r\n");
                        }
                        else if ( reportingType == REPORT_LISTING )
                        {
                            out.write(getString(nlsBundle,"LIST.TASK.NAME") + task.getTitle(locale) + "\r\n");
                            out.write(getString(nlsBundle,"LIST.TASK.OID") + Long.toHexString(task.ObjectID()).toUpperCase() + "\r\n");
                            out.write(getString(nlsBundle,"LIST.TASK.ID") + task.getID() + "\r\n\n");
                        }
                    }
                }
            }
        }
        catch ( InvalidObjectIDException ioid )
        {   // Just skip object
        }
        
        return ICliBundle.OK;
    }
    
    /**
     * Cli command to list all of the available groups.  This command is offered
     * to display the available group ids which could be used with modifyUserAttributes.
     * Argument Input Format: [-r|-report|-t|-terse]
     */
    public int listGroups(Locale locale, String [] args, Writer out) throws Exception
    {
        if ( nlsBundle == null )
            nlsBundle = getBundle(LanguageBundle, locale);

        setReportingLevel(args);

        try
        {   // Get all of the TWGFilter objects
            Enumeration objs = TWGObject.AllObjects( Class.forName("com.tivoli.twg.engine.TWGFilter") );

            while ( objs.hasMoreElements() )
            {
                TWGFilter filter = (TWGFilter)objs.nextElement();
                // Skip these group type(s)
                if ( filter.isNone() )
                {
                    continue;
                }

                if ( reportingType == TERSE_LISTING )
                {
                    out.write( Long.toHexString(filter.ObjectID()).toUpperCase() + "\r\n");
                }
                else if ( reportingType == STANDARD_LISTING )
                {
                    out.write( Long.toHexString(filter.ObjectID()).toUpperCase() + "\t" +  filter.getName(locale) + "\r\n");
                }
                else if ( reportingType == REPORT_LISTING )
                {
                    out.write(getString(nlsBundle,"LIST.FILTER.NAME") + filter.getName(locale) + "\r\n");
                    out.write(getString(nlsBundle,"LIST.FILTER.OID") + Long.toHexString(filter.ObjectID()).toUpperCase() + "\r\n");
                    out.write(getString(nlsBundle,"LIST.FILTER.ID") + getString(nlsBundle,"GRPTYPE." + filter.getType()) + "\r\n\n");
                }
            }
        }
        catch ( InvalidObjectIDException ioid )
        {
        }
        return ICliBundle.OK;
    }
    
    /**
     * Cli command to add a group to a user's group access list.   
     * Argument Input Format: <user> [-id|-name] <group1> <group2> ... <groupN>
     */
    public int addGroupAccess(Locale locale, String [] args, Writer out) throws Exception
    {
        if ( nlsBundle == null )
            nlsBundle = getBundle(LanguageBundle, locale);

        if ( args.length < 2 )
        {
            out.write(getString(nlsBundle, "ERROR.SYNTAX") + "\r\n");
            out.write(getString(nlsBundle, "CLI_COMMANDS.9") + "\r\n");
            return ICliBundle.USAGE;
        }

        int argIndex = 0;
        // Try to get TWGUser object.  First arg is user id
        TWGUser user = findTWGUser( args[argIndex] );

        if (user == null)
        {
            out.write(getString(nlsBundle, "ERROR.USER_NOT_FOUND") + "\r\n");
            return ICliBundle.USAGE;
        }
        // Check if user is a SuperUser.  SuperUsers can't be modified.
        if ( UserIDSecurity.isSuperUser(user.getName()) )
        {
            out.write(getString(nlsBundle, "ERROR.SUPERUSER") + "\r\n");
            return ICliBundle.USAGE;
        }
        argIndex++;
        boolean addByOID = true;
        // Look for switch to determine if groups were added by Name or OID.
        if (args[argIndex].startsWith("-") )
        {
            if (args[argIndex].startsWith(NAME_SWITCH))
            {
                addByOID = false;
            }
            argIndex++;
        }
        // Add each group to user
        for (int i = argIndex; i < args.length; i++)
        {
            long foid = findGroupOID(args[i], locale, addByOID);
            if (foid != Long.MIN_VALUE)
            {
                user.addFilterToFilterAccessList(foid);
                //If user currently has all access, restrict access
                if ( !user.isFilterLimited() )
                {
                    setAttributeValue(user, locale, UserCli.ATTRIB_GROUP_FLAG, 
                                      new TWGInteger(TWGUserConstants.ACCESS_FLAG_LIMIT_FILTERS) );
                }
            }
            else
            {
                out.write(getString(nlsBundle, "ERROR.GROUP_NOT_FOUND") + ":" + args[i] + "\r\n" );
            }
        }

        // If we changed any values than we need to save the TWGUser object
        try
        {
            if ( user.isSaveRequired() )
                user.save();
        }
        catch ( TWGPersistentObjectSaveException pos )
        {
            out.write(getString(nlsBundle, "ERROR.SAVE_FAILED") + "\r\n");
            return ICliBundle.COMMAND_EXCEPTION;
        }

        out.write( "\r\n" + formatGroupsListString(user, locale) );
        return ICliBundle.OK;

    }


    /**
     * Cli command to remove a group to a user's group access list.   
     * Argument Input Format: <user> [-name|-id] <group1> <group2> ... <groupN>
     */
    public int removeGroupAccess(Locale locale, String [] args, Writer out) throws Exception
    {
        if ( nlsBundle == null )
            nlsBundle = getBundle(LanguageBundle, locale);

        if ( args.length < 2 )
        {
            out.write(getString(nlsBundle, "ERROR.SYNTAX") + "\r\n");
            out.write(getString(nlsBundle, "CLI_COMMANDS.10") + "\r\n");
            return ICliBundle.USAGE;
        }

        int argIndex = 0;
        // Try to get TWGUser object.  First arg is user id
        TWGUser user = findTWGUser( args[argIndex] );

        if (user == null)
        {
            out.write(getString(nlsBundle, "ERROR.USER_NOT_FOUND"));
            return ICliBundle.USAGE;
        }

        // Check if user is a SuperUser.  SuperUsers can't be modified.
        if ( UserIDSecurity.isSuperUser(user.getName()) )
        {
            out.write(getString(nlsBundle, "ERROR.SUPERUSER") + "\r\n");
            return ICliBundle.USAGE;
        }

        argIndex++;
        boolean addByOID = true;
        // Look for switch to determine if groups were added by Name or OID.
        if (args[argIndex].startsWith("-") )
        {
            if (args[argIndex].startsWith(NAME_SWITCH))
            {
                addByOID = false;
            }
            argIndex++;
        }
        // Get current filter access list for this user
        LongValueSet filterList = new LongValueSet( user.getFilterAccessList() );

        // Remove each group from user list if it is currently in the list
        for (int i = argIndex; i < args.length; i++)
        {
            long foid = findGroupOID(args[i], locale, addByOID);
            if (foid != Long.MIN_VALUE)
            {
                if ( !filterList.RemoveValue(foid))
                    out.write(getString(nlsBundle, "ERROR.GROUP_NOT_FOUND") + "\r\n");
            }
            else
            {
                out.write(getString(nlsBundle, "ERROR.GROUP_NOT_FOUND") + "\r\n");
            }
        }
        // Set the filter list to the new list
        user.setFilterAccessList( filterList.GetValues() );
        
        // If we changed any values than we need to save the TWGUser object
        try
        {
            if ( user.isSaveRequired() )
                user.save();
        }
        catch ( TWGPersistentObjectSaveException pos )
        {
            out.write(getString(nlsBundle, "ERROR.SAVE_FAILED") + "\r\n");
            return ICliBundle.COMMAND_EXCEPTION;
        }

        out.write( "\r\n" + formatGroupsListString(user, locale) );

        return ICliBundle.OK;

    }

    /**
     * Cli command to add a task to a user's task access list.   
     * Argument Input Format: <user> [-name|-id] <task1> <task2> ... <taskN>
     */
    public int addTaskAccess(Locale locale, String [] args, Writer out) throws Exception
    {
        if ( nlsBundle == null )
            nlsBundle = getBundle(LanguageBundle, locale);

        if ( args.length < 2 )
        {
            out.write(getString(nlsBundle, "ERROR.SYNTAX") + "\r\n");
            out.write(getString(nlsBundle, "CLI_COMMANDS.11") + "\r\n");
            return ICliBundle.USAGE;
        }
        
        int argIndex = 0;
        // Try to get TWGUser object.  First arg is user id
        TWGUser user = findTWGUser( args[argIndex] );
        
        if (user == null)
        {
            out.write(getString(nlsBundle, "ERROR.USER_NOT_FOUND") + "\r\n");
            return ICliBundle.USAGE;
        }

        // Check if user is a SuperUser.  SuperUsers can't be modified.
        if ( UserIDSecurity.isSuperUser(user.getName()) )
        {
            out.write(getString(nlsBundle, "ERROR.SUPERUSER") + "\r\n");
            return ICliBundle.USAGE;
        }
        
        argIndex++;
        boolean addByOID = true;
        // Look for switch to determine if groups were added by Name or OID.
  
        if (args[argIndex].startsWith("-") )
        {
            if (args[argIndex].startsWith(NAME_SWITCH))
            {
                addByOID = false;
            }
            argIndex++;
        }

        // Add each task to user
        for (int i = argIndex; i < args.length; i++)
        {
            long toid = findTaskOID(args[i], addByOID);
            if (toid != Long.MIN_VALUE)
            {
                user.addTaskToTaskAccessList(toid);
                //If user currently has all access, restrict access
                if ( !user.isTaskLimited() )
                {
                    setAttributeValue(user, locale, UserCli.ATTRIB_TASK_FLAG,
                                      new TWGInteger(TWGUserConstants.ACCESS_FLAG_LIMIT_TASKS) );
                }
            }
            else
            {
                out.write(getString(nlsBundle, "ERROR.TASK_NOT_FOUND") + ":" + args[i] + "\r\n" );
            }
        }
        // If we changed any values than we need to save the TWGUser object
        try
        {
            if ( user.isSaveRequired() )
                user.save();
        }
        catch ( TWGPersistentObjectSaveException pos )
        {
            out.write(getString(nlsBundle, "ERROR.SAVE_FAILED") + "\r\n");
            return ICliBundle.COMMAND_EXCEPTION;
        }

        out.write( "\r\n" + formatTasksListString(user, locale) );
        
        return ICliBundle.OK;

    }


    /**
     * Cli command to remove a task to a user's task access list.   
     * Argument Input Format: <user> [-name|-id] <task1> <task2> ... <taskN>
     */
    public int removeTaskAccess(Locale locale, String [] args, Writer out) throws Exception
    {
        if ( nlsBundle == null )
            nlsBundle = getBundle(LanguageBundle, locale);

        if ( args.length < 2 ) 
        {
            out.write(getString(nlsBundle, "ERROR.SYNTAX") + "\r\n");
            out.write(getString(nlsBundle, "CLI_COMMANDS.12") + "\r\n");
            return ICliBundle.USAGE;
        }

        int argIndex = 0;
        // Try to get TWGUser object.  First arg is user id
        TWGUser user = findTWGUser( args[argIndex] );

        if (user == null)
        {
            out.write(getString(nlsBundle, "ERROR.USER_NOT_FOUND"));
            return ICliBundle.USAGE;
        }

        // Check if user is a SuperUser.  SuperUsers can't be modified.
        if ( UserIDSecurity.isSuperUser(user.getName()) )
        {
            out.write(getString(nlsBundle, "ERROR.SUPERUSER") + "\r\n");
            return ICliBundle.USAGE;
        }

        argIndex++;
        boolean addByOID = true;
        // Look for switch to determine if groups were added by Name or OID.
        if (args[argIndex].startsWith("-") )
        {
            if (args[argIndex].startsWith(NAME_SWITCH))
            {
                addByOID = false;
            }
            argIndex++;
        }
        
        // Get current task access list for this user
        LongValueSet taskList = new LongValueSet( user.getTaskAccessList() );

        // Remove each group from user list if it is currently in the list
        for (int i = argIndex; i < args.length; i++)
        {
            long toid = findTaskOID(args[i], addByOID);
            if (toid != Long.MIN_VALUE)
            {
                if ( !taskList.RemoveValue(toid))
                    out.write(getString(nlsBundle, "ERROR.TASK_NOT_FOUND") + "\r\n");
            }
            else
            {
                out.write(getString(nlsBundle, "ERROR.TASK_NOT_FOUND") + "\r\n");
            }
        }
        // Set the filter list to the new list
        user.setTaskAccessList( taskList.GetValues() );
        // If we changed any values than we need to save the TWGUser object
        try
        {
            if ( user.isSaveRequired() )
                user.save();
        }
        catch ( TWGPersistentObjectSaveException pos )
        {
            out.write(getString(nlsBundle, "ERROR.SAVE_FAILED") + "\r\n");
            return ICliBundle.COMMAND_EXCEPTION;
        }

        out.write( "\r\n" + formatTasksListString(user, locale) );

        return ICliBundle.OK;
    }

    /**
    * Look at command line argument and set the reporting level for listings
    * Method sets instance variable reportingType
    * @param String[] args - CLI arguments
    */
    private void setReportingLevel(String[] args)
    {
        reportingType = STANDARD_LISTING;
        if ( args.length > 0 )
        {
            if ( args[0].startsWith(REPORT_SWITCH) )
            {
                reportingType = REPORT_LISTING;
            }
            else if ( args[0].startsWith(TERSE_SWITCH) )
            {
                reportingType = TERSE_LISTING;
            }
        }
    }
    /**
     * Private method to parse the remaining command line into attributes and values which
     * can be used by modifyUserAttributes.
     * @param args - Command line input arguments
     * @param out - Output writer for error messages
     * @return - array of AttributeValuePair object for the attributes in userAttributesList
     */ 
    private AttributeValuePair [] parseAttributeArguments(final String [] args, Writer out) throws Exception
    {
        String currentAttrib = null;     //Keeps track of which attribute the value goes with
        AttributeValuePair avp = null;
        // Clear any values out of the set
        AttributeValuePair.clear();

        // Attributes start at arg[1]
        for ( int i = 1; i < args.length; i++ )
        {
            int sep = args[i].indexOf('=');
            if ( sep >= 1 )  // User entered ATTRIB=VALUE or ATTRIB= VALUE
            {
                String attrib = UserCli.UserAttribPrefix.concat( args[i].substring(0,sep).toLowerCase() );
                // Check if this is one of our attributes
                if ( validateAttributeInput(attrib) )
                {
                    String value = args[i].substring(sep+1);

                    if ( value.length() > 0 || ( i == (args.length - 1) ) )
                    {   // If we have a value, clean it up and create a AttributeValuePair
                        if ( value.startsWith("{") )
                            value = value.substring(1, value.length() );

                        if ( value.endsWith("}") || value.endsWith(",") )
                            value = value.substring(0, value.length() - 1 );

                        AttributeValuePair.makeAttributeValuePair(attrib, value);
                    }
                    else if ( (i != (args.length-1))  && (args[i+1].indexOf('=') >= 1) )
                    {   //If the next value is another attribute, add attribute with empty value string
                        AttributeValuePair.makeAttributeValuePair(attrib,"");
                    }

                    if ( args[i].endsWith("}") )   
                        currentAttrib = null;       //If the value was the end of a set, we are done with this attribute
                    else
                        currentAttrib = attrib;     //If not, additional args may contain more values, remember this attribute
                }
                else
                {
                    out.write(getString(nlsBundle,"ERROR.INVALID_ATTRIB") + ": " + attrib + "\r\n");
                }

            }
            else if ( (sep == -1) && (currentAttrib == null) )
            {   // We must have a new attribute (ie, ATTRIB = VALUE with args[i] pointing to ATTRIB).
                String attrib = UserCli.UserAttribPrefix.concat( args[i].trim().toLowerCase() );

                if ( validateAttributeInput(attrib) )
                {
                    //Remember which attribute we are working on so we can add it when we get the value.
                    currentAttrib = attrib;
                }
                else
                {
                    out.write(getString(nlsBundle,"ERROR.INVALID_ATTRIB") + ": " + attrib + "\r\n");
                }
            }
            else if ( (sep == -1) && (currentAttrib != null) )
            {   // We either have a single value (ie, ATTRIB = VALUE with args[i] pointing to VALUE), or
                // we have a set: { aa, bb,cc ,dd} and we are somewhere in the set.
                String value = args[i];

                // Strip off last comma or close bracket
                if ( value.endsWith(",") || value.endsWith("}") )
                    value = value.substring(0, value.length() - 1 );

                // Strip off first comma or open bracket
                if ( value.startsWith("{") || value.startsWith(",") )
                    value = value.substring(1, value.length() );

                AttributeValuePair.makeAttributeValuePair(currentAttrib, value);

                // If this was the end of a set, set the current attribute we are working with to null
                if ( args[i].endsWith("}") )
                    currentAttrib = null;
            }
        }
        return AttributeValuePair.getAllValues();
    }
    /**
     * Private method to set the attribute value on a particular user.
     * @param user - TWGUser to modify
     * @param locale - Locale of cli client
     * @param attrib - String attribute to modify
     * @param value - DataValue to set attribute
     * @return boolean - true if set works, false if not
     */ 

    private boolean setAttributeValue(TWGUser user, Locale locale, String attrib, DataValue value)
    {
        boolean success = false;
        String attribute = attrib;
        DataValue newValue = value;

        // Access flags require special processing before doing the set.
        if ( attrib.equals(UserCli.ATTRIB_GROUP_FLAG) || attrib.equals(UserCli.ATTRIB_READ_ONLY_FLAG) || 
             attrib.equals(UserCli.ATTRIB_TASK_FLAG) )
        {
            // Use generic ACCESSFLAGS attribute
            attribute = TWGUserConstants.ATTRIB_ID_ACCESSFLAGS;
            // Need to get the current flags so that we preserve bits that we aren't explicitly setting
            TWGInteger currentAccessFlags = (TWGInteger)user.getAttributeValue(TWGUserConstants.ATTRIB_ID_ACCESSFLAGS, locale);

            int newFlags = ((TWGInteger)value).getValue();

            if ( newFlags == 0 )
            {   // Turn off the appropriate bit only
                if ( attrib.equals(UserCli.ATTRIB_GROUP_FLAG) )
                {
                    newFlags = currentAccessFlags.getValue() & ~(TWGUserConstants.ACCESS_FLAG_LIMIT_FILTERS);
                }
                else if ( attrib.equals(UserCli.ATTRIB_TASK_FLAG) )
                {
                    newFlags = currentAccessFlags.getValue() & ~(TWGUserConstants.ACCESS_FLAG_LIMIT_TASKS);
                }
                else if ( attrib.equals(UserCli.ATTRIB_READ_ONLY_FLAG) )
                {
                    newFlags = currentAccessFlags.getValue() & ~(TWGUserConstants.ACCESS_FLAG_RDONLY_FILTERS);
                }
                // Create a new DataValue with the correct bits
                newValue = new TWGInteger(newFlags);
            }
            else
            {   // Turn on the appropriate bit
                newValue = new TWGInteger(currentAccessFlags.getValue() | newFlags);
            }
        }
        // Set the value on the TWGUser object
        success = user.setAttributeValue(attribute, newValue);

        return success;
    }
    /**
     * Simple helper method to verify that the attribute that was entered is
     * one of the settable attributes listed in userAttributesList
     * @param attribute - String name of attribute
     * @return boolean - true if found
     */
    private boolean validateAttributeInput(String attribute)
    {
        // Search through the userAttributeList array and look for a match
        for ( int i = 0; i < userAttributesList.length; i++ )
        {
            if ( attribute.equals(userAttributesList[i]) )
                return true;
        }
        // If not found, return false
        return false;
    }

    /**
     * Get the TWGUser object by the user id.  If the user does not exist and the
     * user is a member of the appropriate groups, create a new TWGUser object and
     * return it.
     * @param userName - user id string
     * @return TWGUser object with that id
     */
    private TWGUser findTWGUser(String userName)
    {
        boolean isNativeOnlyUser = false;

        // Try to get TWGUser object
        TWGUser user = TWGUser.find( userName );

        // Check if user exists
        if ( user == null )
        {
            // If no TWGUser, check for native user id
            String [] allNativeUsers = UserIDSecurity.getNativeUserIDs();
            int i = 0;
            while ( (!isNativeOnlyUser) && (i < allNativeUsers.length) )
            {
                if ( userName.equals(allNativeUsers[i]) )
                {
                    isNativeOnlyUser = true;
                }
                i++;
            }
        }
        // If it is a native user with no TWGUser object, create a new TWGUser
        if ( isNativeOnlyUser )
        {
            try
            {
                user = new TWGUser(userName, "");
                user.save();
            }
            catch ( TWGPersistentObjectSaveException pose )
            {
                user = null;
            }
        }

        return user;

    }
    /**
     * Get the correct group OID given a string with either the group name
     * or group oid.
     * @param group - String of either group name or group oid
     * @param locale - Locale for group name
     * @param byOID - true if the string is a group oid
     * @return long - group OID
     */
    private long findGroupOID(String group, Locale locale, boolean byOID)
    {
        long foid = Long.MIN_VALUE;
        boolean success = false;

        if (byOID)  
        {   
            try
            {   // Find the group by the OID string
                long oid = Long.parseLong( (String)group, 16);
                TWGObject obj = TWGObject.FindObject(oid);
                if ( (obj != null) && (obj instanceof TWGFilter) )
                {
                    foid = oid;
                }
            }
            catch ( NumberFormatException nfe )
            {   // Ignore the entry.  It is not a valid long
            }
        }
        else 
        {   // User added group by name.  Attempt to find TWGFilter object
            TWGFilter[] groups = TWGFilter.findByName(group, locale, null);

            if ( (groups.length > 0) && (groups[0] != null) )
            {
                try {
                    foid = groups[0].ObjectID();
                } catch (InvalidObjectIDException ide) {
                    // Ignore the entry
                }
            } 
        }
        return foid;
    }
    
    /**
     * Get the correct task OID given a string with either the task name
     * or task oid.
     * @param task - String of either task name or task oid
     * @param byOID - true if the string is a task oid
     * @return long - task OID
     */
    private long findTaskOID(String task, boolean byOID)
    {
        long toid = Long.MIN_VALUE;
        boolean success = false;

        if (byOID)  
        {   
            try
            {   // Find the group by the OID string
                long oid = Long.parseLong( (String)task, 16);
                TWGObject obj = TWGObject.FindObject(oid);
                if ( (obj != null) && (obj instanceof TWGTask) )
                {
                    TWGTask tsk = (TWGTask)obj;
                    if ( (!tsk.isUnrestricted()) && (!tsk.isDisabled()) && (tsk.getParent() == null) )
                    {
                        toid = oid;
                    }
                }
            }
            catch ( NumberFormatException nfe )
            {   // Ignore the entry.  It is not a valid long
            }
        }
        else 
        {   // User added group by name.  Attempt to find TWGFilter object
            TWGTask tsk = TWGTask.getTask(task);

            if ( (tsk != null) && (!tsk.isUnrestricted()) && (!tsk.isDisabled()) && (tsk.getParent() == null) )
            {
                try {
                    toid = tsk.ObjectID();
                } catch (InvalidObjectIDException ide) {
                    // Ignore the entry
                }
            } 
        }
        return toid;
    }

    /**
     * load NLS messages
     */
    private ResourceBundle getBundle(String cls, Locale locale)
    {

        ResourceBundle resBundle = null;

        try
        {
            resBundle = ResourceBundle.getBundle(cls, locale);
        }
        catch ( Exception e )
        {

            Locale defaultLocale = new Locale(new String("en"), new String("US"));
            resBundle = ResourceBundle.getBundle(cls, defaultLocale);
        }
        return resBundle;
    }
    /**
     * get NLS String
     */
    private String getString(ResourceBundle nlsBundle, String s) 
    {
        String nlsString = s;
        try
        {
            nlsString = nlsBundle.getString(s);
        }
        catch ( MissingResourceException mre )
        {
            nlsString = s;
        }

        return nlsString;

    }

    private String formatGroupsListString( TWGUser usr, Locale locale )
    {
        if ( nlsBundle == null )
            nlsBundle = getBundle(LanguageBundle, locale);

        StringBuffer groupListString = new StringBuffer();
        // If user has access to all groups, display ALL GROUPS
        if ( !usr.isFilterLimited() )
        {
            groupListString.append( getString(nlsBundle,"LIST.USER.GROUPS") );
            groupListString.append( getString(nlsBundle,"ALLGROUPS") );
            if ( usr.isReadOnlyFilterAccess() ) 
            {
                groupListString.append( " (" );
                groupListString.append( getString(nlsBundle, "READONLY" ) ).append( ") " );
            }
            groupListString.append("\r\n");
        }
        else
        {   // Get group access list for this user
            LongValueSet filters = usr.getFilterAccessList();

            groupListString.append(getString(nlsBundle,"LIST.USER.GROUPS") );
            // If user has read only access to list, display (READONLY) in front of group info.
            if ( usr.isReadOnlyFilterAccess() )
            {
                groupListString.append( "(" );
                groupListString.append( getString(nlsBundle, "READONLY" ) ).append( ") " );
            }
            // Print each group OID comma separated.
            groupListString.append("{ ");
            if ( (filters != null) && (filters.Length() > 0 ) )
            {
                for ( int j = 0; j < filters.Length(); j++ )
                {
                    TWGFilter filter = (TWGFilter)TWGObject.FindObject( filters.GetValue(j) );
                    if ( filter != null )
                    {
                        groupListString.append( filter.getName(locale) ).append( " (" );
                        groupListString.append( Long.toHexString(filters.GetValue(j)).toUpperCase() ).append( ")" );
                        if ( j != filters.Length() - 1 )
                        {
                            groupListString.append(", ");
                        }
                    }
                }
            }
            groupListString.append(" }\r\n");
        }
        return groupListString.toString();
    }


    private String formatTasksListString( TWGUser usr, Locale locale )
    {
        if ( nlsBundle == null )
            nlsBundle = getBundle(LanguageBundle, locale);

        StringBuffer taskListString = new StringBuffer();

        // If user has access to all tasks, display ALL TASKS
        if ( !usr.isTaskLimited() )
        {
            taskListString.append( getString(nlsBundle, "LIST.USER.TASKS") );
            taskListString.append( getString(nlsBundle, "ALLTASKS")).append("\r\n") ;
        }
        else
        {   
            taskListString.append( getString(nlsBundle,"LIST.USER.TASKS") ).append("{ ");
            // Get user task access list
            LongValueSet tasks = usr.getTaskAccessList();
            if ( ( tasks != null ) && ( tasks.Length() > 0 ) )
            {
                // Display all task OIDs comma separated
                for ( int j=0; j < tasks.Length(); j++ )
                {
                    TWGTask task = TWGTask.getTask( tasks.GetValue(j) );
                    if ( task != null )
                    {
                        taskListString.append( task.getID() ).append( " (");
                        taskListString.append( Long.toHexString(tasks.GetValue(j)).toUpperCase() ).append( ")" );
                        if ( j != tasks.Length() - 1 )
                        {
                            taskListString.append(", ");
                        }
                    }
                }
            }
            taskListString.append(" }\r\n");
        }
        return taskListString.toString();
    }
}


/**
 * AttributeValuePair
 * Class to hold parsed attribute and data value from command line.  This
 * class allows multiple values to be accumulated for a single attribute.
 */
class AttributeValuePair
{
    private static HashMap pairs = new HashMap();  // Static hash of all attribute-value pairs
    String attribute;                              // Attribute label
    DataValue dv;                                  // DataValue

    /**
     * AttributeValuePair constructor
     * Private constructor for creating a new AttributeValuePair.
     * Objects should be created using static method makeAttributeValuePair
     */
    private AttributeValuePair(String attrib, String val)
    {
        attribute = attrib;
        dv = parseDataValues(val);
    }

    /**
     * Method to initialize AttributeValuePair.  If no object with specified
     * attribute exists, a new pair will be created.  If there is an existing
     * attribute, the value will be appended according to the rules for that
     * attribute.
     * @param - attribute
     * @param - value
     */
    public static void makeAttributeValuePair(String attrib, String val)
    {
        if ( attrib == null )
            throw new IllegalArgumentException();

        AttributeValuePair avp = null;
        // Find an existing object for this attribute
        avp = (AttributeValuePair)pairs.get(attrib);
        if ( avp == null )
        {
            avp = new AttributeValuePair(attrib,val);
            pairs.put(attrib, avp);
        }
        else
        {
            avp.appendDataValue(val);
        }
        // Special case.  If the attribute is a new task list or group list, we
        // also need to have an attribute to set the corresponding access flag. 
        if ( attrib.equals(UserCli.ATTRIB_TASK_ACCESS_LIST) )
        {
            makeAttributeValuePair(UserCli.ATTRIB_TASK_FLAG,"true");
        }
        if ( attrib.equals(UserCli.ATTRIB_FILTER_ACCESS_LIST) )
        {
            makeAttributeValuePair(UserCli.ATTRIB_GROUP_FLAG,"true");
        }

    }
    /**
     * Returns the attribute for this pair
     * @returns attribute
     */
    public String getAttribute()
    {
        return attribute;
    }

    /**
     * Retrieves all of the AttributeValuePairs
     * @returns Array of AttributeValuePairs
     * 
     */
    public static AttributeValuePair [] getAllValues()
    {
        AttributeValuePair [] values = new AttributeValuePair[pairs.size()];

        Collection c = pairs.values();

        return(AttributeValuePair [])c.toArray(values);;
    }

    /**
     * Retrieves the value for this pair
     * @return Datavalue
     */
    public DataValue getDataValue()
    {
        return dv;
    }
    /**
     * Removes all AttributeValuePair objects
     * 
     */
    public static void clear()
    {
        pairs.clear();
    }
    /**
     * Parses the string argument that was entered and creates the appropriate
     * DataValue based on the attribute.  Parsing assumes that values are comma
     * separated
     * @param val - String of values
     * @return DataValue - values converted to appropriate DataValue
     */
    private DataValue parseDataValues(String val)
    {
        DataValue dv = null;
        ArrayList tokenList = new ArrayList();
        // Divide string argument by commas
        StringTokenizer st = new StringTokenizer(val, ",");
        // Put the arguments into the tokenList
        while ( st.hasMoreTokens() )
        {
            tokenList.add(st.nextToken().trim());
        }
        // Create the appropriate DataValue based on the attribute.
        if ( attribute.equals(UserCli.ATTRIB_EMAIL) || attribute.equals(UserCli.ATTRIB_PAGER) )
        {
            if (tokenList.size() > 0)
            {
                String value = (String)tokenList.get( tokenList.size() - 1 );
                dv = new TWGString(value);
            }
            else
            {
                dv = new TWGString("");
            }
        }
        else if ( attribute.equals(UserCli.ATTRIB_PRIVSET) )
        {
            // Verify that all tokens in list are actually privilege tokens
            ListIterator iter = tokenList.listIterator();
            while ( iter.hasNext() )
            {
                String privToken = (String)iter.next();
                if ( !TWGPrivToken.isPrivTokenDefined(privToken) )
                {   // If no such token, remove from list
                    iter.remove();
                }
            }
            // Convert token list to array of privilege token strings
            String [] privs = (String [])tokenList.toArray(new String[0]);

            dv = new TWGStringArray(privs);
        }
        else if ( attribute.equals(UserCli.ATTRIB_GROUP_FLAG) )
        {
            if ( Boolean.valueOf((String)tokenList.get(0)) == Boolean.TRUE )
            {
                int access = TWGUserConstants.ACCESS_FLAG_LIMIT_FILTERS;
                dv = new TWGInteger(access);
            }
            else
                dv = new TWGInteger(0);
        }
        else if ( attribute.equals(UserCli.ATTRIB_READ_ONLY_FLAG) )
        {
            if ( Boolean.valueOf((String)tokenList.get(0)) == Boolean.TRUE )
            {
                int access = TWGUserConstants.ACCESS_FLAG_RDONLY_FILTERS;
                dv = new TWGInteger(access);
            }
            else
                dv = new TWGInteger(0);
        }
        else if ( attribute.equals(UserCli.ATTRIB_TASK_FLAG) )
        {
            if ( Boolean.valueOf((String)tokenList.get(0)) == Boolean.TRUE )
            {
                int access = TWGUserConstants.ACCESS_FLAG_LIMIT_TASKS;
                dv = new TWGInteger(access);
            }
            else
                dv = new TWGInteger(0);
        }
        else if ( attribute.equals(UserCli.ATTRIB_TASK_ACCESS_LIST) || attribute.equals(UserCli.ATTRIB_FILTER_ACCESS_LIST) )
        {
            // Need to make sure that the arguments are actually oids for tasks or filters
            LongValueSet lvs = new LongValueSet();
            Iterator iter = tokenList.iterator();
            while ( iter.hasNext() )
            {
                try
                {
                    long oid = Long.parseLong( (String)iter.next(), 16);
                    TWGObject obj = TWGObject.FindObject(oid);
                    if ( obj != null )
                    {
                        if ( attribute.equals(UserCli.ATTRIB_FILTER_ACCESS_LIST) && (obj instanceof TWGFilter)  )
                        {
                            lvs.InsertValue( oid );
                        } 
                        else if ( attribute.equals(UserCli.ATTRIB_TASK_ACCESS_LIST) && (obj instanceof TWGTask) )
                        {
                            TWGTask tsk = (TWGTask)obj;
                            if ( (!tsk.isUnrestricted()) && (!tsk.isDisabled()) && (tsk.getParent() == null) )
                            {
                                lvs.InsertValue( oid );
                            }
                        }

                    }
                }
                catch ( NumberFormatException nfe )
                {
                    // Ignore the entry.  It is not a valid long
                }
            }
            dv = new TWGLongArray(lvs);

        }
        else
        {   // If it doesn't match any known attributes, just return a TWGString.
            String value = (String)tokenList.get( tokenList.size() - 1 );
            dv = new TWGString(value);
        }

        return dv;


    }
    /**
     * In the case where a AttributeValuePair already exists, the new value will be appended
     * to the existing DataValue.  In some cases, the new value(s) will simply replace the old
     * value(s).  This is the case if the user entered the same attribute twice or entered multiple
     * values for a single value attribute.  In cases where the attribute is a list, the new
     * value(s) will be added to the list.
     * @param newValue String to be appended
     * 
     */
    private void appendDataValue(String newValue)
    {
        // Trim any commas or brackets
        String addValue = new String(newValue);
        if ( addValue.endsWith(",") || addValue.endsWith("}") )
            addValue = addValue.substring(0, addValue.length() - 1);

        if ( addValue.startsWith(",") || addValue.startsWith("{") )
            addValue = addValue.substring(1, addValue.length() );

        // Parse the string value into a DataValue
        DataValue newDv = parseDataValues(addValue);

        // New Datavalue replaces old value for email, pager and access flags
        if ( attribute.equals(UserCli.ATTRIB_EMAIL) || attribute.equals(UserCli.ATTRIB_PAGER) ||
             attribute.equals(UserCli.ATTRIB_GROUP_FLAG) || attribute.equals(UserCli.ATTRIB_READ_ONLY_FLAG) ||
             attribute.equals(UserCli.ATTRIB_TASK_FLAG) )
        {
            dv = newDv;
        }
        else if ( attribute.equals(UserCli.ATTRIB_PRIVSET) )         // Add the new privileges to the old list
        {
            String [] oldPrivs = ((TWGStringArray)dv).getValue();
            String [] newPrivs = ((TWGStringArray)newDv).getValue();

            String [] privs = new String[oldPrivs.length + newPrivs.length];
            System.arraycopy(oldPrivs, 0, privs, 0, oldPrivs.length);
            System.arraycopy(newPrivs, 0, privs, oldPrivs.length, newPrivs.length);

            dv = new TWGStringArray(privs);
        }
        else if ( attribute.equals(UserCli.ATTRIB_TASK_ACCESS_LIST) || attribute.equals(UserCli.ATTRIB_FILTER_ACCESS_LIST) )
        {
            long [] oldOids = ((TWGLongArray)dv).getValue();
            long [] newOids = ((TWGLongArray)newDv).getValue();

            long [] oids = new long[oldOids.length + newOids.length];
            System.arraycopy(oldOids, 0, oids, 0, oldOids.length);
            System.arraycopy(newOids, 0, oids, oldOids.length, newOids.length);

            dv = new TWGLongArray(oids);

        }
        else
        {   // If the attribute is unknown, just replace the value.
            dv = newDv;
        }
    }

}
