/*****************************************************************************
 * Licensed Materials - Property of IBM                            
 * PID (5697-TKT)                                                             
 * (C) Copyright by IBM 1999. All Rights Reserved.
 ****************************************************************************/
/**
 * This file implements the DLL entry points for a Monitor Library. The function
 * implementations are meant to be generic for all Monitor Libraries.
 */
 
#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
//#include <monitor.h>
#include <mon_sdk.h>

#include <fstream.h>
#include <keyinfob.hpp>
#include <mvalugen.hpp>
#include <monitlib.hpp>

/*
 *  global variables
 */
HINSTANCE               
  hInst;                  // DLL instance
static MonitorLibrary    
 *fMonitor;

/*
 * System functions
 */
BOOL WINAPI DllMain(HINSTANCE hMod, DWORD dwReason, LPVOID plvReserved)
{
#ifdef _DEBUG
  ofstream to("C:\\temp\\test.out", ios::out | ios::app );
  to << "DLLMain" << endl;
#endif
   if (dwReason == DLL_PROCESS_ATTACH)
   {
      hInst = hMod;
   }
   return TRUE;
}


/*
 * Exported Monitor DLL Functions
 */

/*
 * InitNodes
 *
 * Initialize the nodes of the MonitorLibary. We create our ValueGenerator and
 * give it to the MonitorLibrary class. Then we add the nodes for our Monitor
 * Library. The Monitor Agent will ask for this information later.
 */
BOOL WINAPI InitNodes()
{
#ifdef _DEBUG
  ofstream to("C:\\temp\\test.out", ios::out | ios::app );
  to << "InitNodes " << endl;
#endif

  BOOL monitorSupported = TRUE;
  // logic here to determine if monitor is supported on this platform

  // Set the default java lang string to english.
  KeyInfoBlock::setDefaultJavaLangString( "en_US");
  // Create a new monitor and populate it with the monitor nodes.
  fMonitor = new MonitorLibrary;

  // The monitor will be responsible for deleting the ValueGenerator when it
  // is no longer needed.
  fMonitor->adoptValueGenerator( new MyValueGenerator );

  // add nodes will call value generator add Nodes. All user code is in
  // ValueGenerator class.
  fMonitor->addNodes();

  return monitorSupported;
}



/*
 * GetNodeList
 *
 * Return the AttributeNodeList for the queriedPath. The MonitorLibrary class
 * knows how to do this from the information setup during initNodes. Basically
 * we are returning the children nodes of the queriedPath.
 */
AttributeNodeList * WINAPI GetNodeList( AttributePath *queriedPath)
{
#ifdef _DEBUG
  ofstream to("C:\\temp\\test.out", ios::out | ios::app );
  to << "GetNodeList " << endl;
#endif
  // check the queried path before we try to use it.
  if (queriedPath == NULL)
  {
    return NULL;
  }
  // To allow dymanic updates of nodes ask if additional nodes need
  // to be added.
  fMonitor->updateNodesFor( queriedPath );
  return fMonitor->orphanAttributeNodeListFor( queriedPath );
}


/*
 * NodeConnect
 *
 * Return a handle and the data type for the specified AttributePath. This 
 * handle will be used in all subsequent queries for this AttributePath.
 */
BOOL WINAPI NodeConnect( AttributePath *pAPath, 
                         ULONG *handle, 
                         BYTE *dataType)
{
#ifdef _DEBUG
  ofstream to("C:\\temp\\test.out", ios::out | ios::app );
  to << "NodeConnect" << endl;
#endif
  BOOL rc = FALSE;

  // init return values to invalid defaults
  *dataType = INVALID_NODEDATA;

  *handle = fMonitor->getKeyFor( pAPath );
  if (*handle != INVALID_HANDLE)
  {
    // For now this is direct access to the keyinfoblock stored in the tree
    // view. This should really be a reference counted access, but not yet.
    // don't delete it!!!!
    KeyInfoBlock *fBlock = fMonitor->getKeyInfoBlockWith( *handle );

    if ( fBlock )
    {
      *dataType = fBlock->getDataType( );
      delete fBlock;
    }
    rc = TRUE;
  }
  return rc;
}


/*
 * GetNodeAttributes
 *
 * Return the attributes for the KeyInfoBlock that corresponds to the handle.
 * The handle was obtained by the call to NodeConnect. The orphaned Node Parms
 * will be free'd in FreeNodeAttributes.
 * 
 */
NodeParms* WINAPI GetNodeAttributes(ULONG handle)
{
#ifdef _DEBUG
  ofstream to("C:\\temp\\test.out", ios::out | ios::app );
  to << "GetNodeAttributes" << endl;
#endif
  NodeParms *nParms = 0;

  // For now this is direct access to the keyinfoblock stored in the tree
  // view. 
  KeyInfoBlock *fBlock = fMonitor->getKeyInfoBlockWith( handle );
  // orphan the node parms, they will be freed in FreeNodeAttributes.
  if ( fBlock )
  {
    nParms = fBlock->orphanNodeParms( );
    delete fBlock;
  }
  return nParms;
}


/*
 * FreeNodeAttributes
 *
 * Frees the attributes orphaned in GetNodeAttributes.
 */
void WINAPI FreeNodeAttributes(NodeParms *nParms)
{
#ifdef _DEBUG
  ofstream to("C:\\temp\\test.out", ios::out | ios::app );
  to << "FreeNodeAttributes" << endl;
#endif
  if (nParms)
  {
    delete [] (BYTE *)nParms;
    nParms = NULL;
  }
}

/*
 * NodeDisconnect
 *
 * Mark the KeyInfoBlock associated with handle as no longer valid.
 */
BOOL WINAPI NodeDisconnect(unsigned long handle)
{
#ifdef _DEBUG
  ofstream to("C:\\temp\\test.out", ios::out | ios::app );
  to << "NodeDisconnect" << endl;
#endif
  // If the KeyInforBlock is found mark it no longer valid.
  KeyInfoBlock *fBlock = fMonitor->getKeyInfoBlockWith( handle );
  if ( fBlock )
  {
    fBlock->setBlockUsage( FALSE );
    delete fBlock;
  }
  return TRUE;
}




/*
 * NodeGetValue
 *
 * Return the NodeData for the KeyInfoBlock represented by handle. The orhaned
 * NodeData will be freed by NodeFreeValue.
 *  
 */
NodeData* WINAPI NodeGetValue(unsigned long handle)
{
#ifdef _DEBUG
  ofstream to("C:\\temp\\test.out", ios::out | ios::app );
  to << "NodeGetValue" << endl;
#endif
  NodeData *nodeData = 0;

  // For now this is direct access to the keyinfoblock stored in the tree
  // view. This should really be a reference counted access, but not yet.
  // don't delete it!!!!
  KeyInfoBlock *fBlock = fMonitor->getKeyInfoBlockWith( handle );
  // create a NodeData with the value for the key. This member function news
  // a NodeData data structure from the information contained in the 
  // KeyInfoBlock. The data structure is then deleted by NodeFreeValue.
  if ( fBlock )
  {
    nodeData = fBlock->orphanNodeData( );
    delete fBlock;
  }
  return nodeData;
}


/*
 * NodeFreeValue
 *
 * Free the orphaned data structured created in NodeGetValue.
 */
void WINAPI NodeFreeValue(NodeData *nd)
{
#ifdef _DEBUG
  ofstream to("C:\\temp\\test.out", ios::out | ios::app );
  to << "NodeFreeValue" << endl;
#endif
  if (nd)
    delete [] (BYTE *)nd;
}

/*
 * NodePulse
 *
 * This function will check the KeyInfoBlock which is represented by handle and
 * determine if it is time for a value to be taken. The base pulse is set on a
 * per KeyInfoBlock basis.
 */
unsigned long WINAPI NodePulse(unsigned long handle, 
                               unsigned long time_since_last)
{
#ifdef _DEBUG
  ofstream to("C:\\temp\\test.out", ios::out | ios::app );
  to << "NodePulse" << endl;
#endif
  time_t currTime(time(NULL));
  time_t diffTime(0);
  unsigned long next_time(0);

  // Note: time_t is measured in seconds. Monitor pulse values are in milli
  // seconds. The using the monitor time member functions be sure to account
  // for the difference in units.

  // For now this is direct access to the keyinfoblock stored in the tree
  // view. 
  KeyInfoBlock *fBlock = fMonitor->getKeyInfoBlockWith( handle );
  // Protect against someone setting back the clock.
  // note: last time will be in seconds.
  if (fBlock )
  {
    diffTime = currTime - fBlock->getLastTime();
    if (diffTime < 0 )
    {
      diffTime = 0;
    }
    if ( ((unsigned long) (diffTime * 1000)) >= (fBlock->getBasePulse( ) ) )
    {
      // a value generator has been registered with Monitor for setting the
      // values for the KeyInfoBlock
      fMonitor->setValue( *fBlock );

      fBlock->setLastTime( currTime );
      if ((time_since_last * 1000) > 2 * (fBlock->getBasePulse() ) )
        // we are done
        next_time = 0;
      else              
        // Otherwise, schedule one at next appropriate time
        next_time = (fBlock->getBasePulse()/1000); 
    }
    delete fBlock;
  }
  return next_time;
}

/*
 * FreeNodeList
 *
 * Delete data structure orphaned by GetNodeList. 
 */
void WINAPI FreeNodeList( AttributeNodeList * NodeList)
{
#ifdef _DEBUG
  ofstream to("C:\\temp\\test.out", ios::out | ios::app );
  to << "FreeNodeList" << endl;
#endif
   // gives us the opportunity to free the node list created by GetNodeList
   if (NodeList)
   {
      delete [NodeList->size] (LPBYTE)NodeList;
      NodeList = NULL;
   }
}

/*
 * TermNodes
 *
 * Called when Moitor Library is terminated. Use to clean up.
 */
BOOL WINAPI TermNodes()
{
#ifdef _DEBUG
  ofstream to("C:\\temp\\test.out", ios::out | ios::app );
  to << "TermNodes" << endl;
#endif
  // called when monitor is terminated. This would free global memory, etc.
  // return code indicates whether or not the termination was successful.
  if (fMonitor)
  {
    delete fMonitor;
  }
  return TRUE;
}

