/*****************************************************************************
 * Licensed Materials - Property of IBM                            
 * PID (5697-TKT)                                                             
 * (C) Copyright by IBM 1999. All Rights Reserved.
 ****************************************************************************/
#include <keyinfob.hpp>
#include <mvalugen.hpp>
#include <monitlib.hpp>
#include <fstream.h>
#include <stdio.h>

#include <time.h>

/**
 * This class is an example of overriding the member functions of ValueGenerator.
 * There are two member functions that need to be overridden, these are 
 * addNodesTo and setValueFor.
 */ 

/**
 * Constructor for the MyValueGenerator classs. The KeyInfoBlocks are kept
 * around so that they can be used for comparisons later. Actually only the 
 * leaf nodes would be needed but this keeps the code consistant.
 */
MyValueGenerator::MyValueGenerator()
: fSampleMonitor( "Sample",
                  "MONSAMP")
, fFolder2( "Folder",
            "FOLDER2")
, fFolder3( "Folder",
            "FOLDER3")
, fExpandableFolder( "Expandable Folder",
                     "EXPANDABLEFOLDER")
, fPI( "PI",
       "PI")
, fTime( "Time",
         "SECONDS")
, fPrinter1( "Printer 1",
             "PRINTER1")
, fPrinter2( "Printer 2",
             "PRINTER2")
{
}

/**
 * Destructor.
 */
MyValueGenerator::~MyValueGenerator()
{
}

/**
 * Determine the value for the KeyInfoBlock and set the value.
 */
void MyValueGenerator::setValueFor( KeyInfoBlock& fBlock )
{
  // Use the KeyInfoBlock created in the ctr to compare with fblock.
  // Then supply the data for the KeyInfoBlock requested.
  if ( fBlock == fPrinter1 )
  {
    // code that actually determines value would be placed here.
    fBlock.setStringValue( "offline" );
    fBlock.setDataValidity( TRUE );
  }
  else if ( fBlock == fPrinter2 )
  {
    fBlock.setStringValue( "online" );
    fBlock.setDataValidity( TRUE );
  }
  else if ( fBlock == fPI )
  {
    fBlock.setDoubleValue( 3.14 );
    fBlock.setDataValidity( TRUE );
  }
  else if ( fBlock == fTime )
  {
    if ( fBlock.getDataType() == KeyInfoBlock::kString )
    {
      time_t aclock;
      time( &aclock );                 /* Get time in seconds */

      /* Convert time to struct */
      /* tm form */
      struct tm *newtime = localtime( &aclock );  

      /* Print local time as a string */
      fBlock.setStringValue( asctime( newtime));
      fBlock.setDataValidity( TRUE );
    }
  }
  else
  {
    // must be one of the children from the expandable folder. Just set the
    // value to its programmatic name.
    fBlock.setStringValue( fBlock.getDisplayableName() );
    fBlock.setDataValidity( TRUE );
  }
}

/**
 * Add the nodes to the passed Monitor Libary Object.
 */
void MyValueGenerator::addNodesTo( MonitorLibrary& fMonitor )
{
  // set up the tree that represents the monitors defined in this dll. This 
  // directed graph will be used to supply the information required in GetNode
  // list. We will then assign a key to look up the key information blocks 
  // during NodeConnect. This key will be used to pulse for values and then
  // supply those values when needed.

  // The first entry is a dummy entry for the root node. The programmatic name of
  // this node needs be the name of the dll.
  fMonitor.addKeyInfoBlock( fSampleMonitor );

  // next level of nodes, two are folders and 1 is a leaf node. fSampleMonitor
  // is the parent of the following nodes.
  fMonitor.addKeyInfoBlock( fSampleMonitor, fFolder2 );

  fMonitor.addKeyInfoBlock( fSampleMonitor, fFolder3 );

  fMonitor.addKeyInfoBlock( fSampleMonitor, fExpandableFolder );
  // Normally the framework can figure out a folder from a leaf node by whether
  // or not it has children. In the case of dynamic nodes were the children are 
  // added later, we need to set the node as a folder now.
  fExpandableFolder.setLeafNode( FALSE );

  // next are all of our leaf nodes.

  // Set up information to be returned with getNodeParms
  fPI.setBasePulse( 30 * 1000 ); // once every 30 secs
  fPI.setPulsesPerData( 1 ); // get data on every pulse.
  fPI.setNumberRecorded( 10 ); // keep around 5 minutes worth.
  fPI.setFormat( "%6.2f");
  fPI.setDataType( KeyInfoBlock::kDouble );
  fMonitor.addKeyInfoBlock( fSampleMonitor, fPI );

  // add pulse information to the leaf nodes.
  fTime.setBasePulse( 15 * 1000 ); // once every 15 secs
  fTime.setPulsesPerData( 1 ); // get data on every pulse.
  fTime.setNumberRecorded( 10 ); // keep around 5 minutes worth.
  fTime.setFormat( "%s");
  fTime.setDataType( KeyInfoBlock::kString );
  fMonitor.addKeyInfoBlock( fFolder2, fTime );

  // add pulse information to the leaf nodes.
  fPrinter1.setBasePulse( 30 * 1000 ); // once every 30 secs
  fPrinter1.setPulsesPerData( 1 ); // get data on every pulse.
  fPrinter1.setNumberRecorded( 10 ); // keep around 5 minutes worth.
  fPrinter1.setFormat( "%s");
  fPrinter1.setDataType( KeyInfoBlock::kString );
  fMonitor.addKeyInfoBlock( fFolder2, fPrinter1 );

  // add pulse information to the leaf nodes.
  fPrinter2.setBasePulse( 20 * 1000 ); // once every 20 secs
  fPrinter2.setPulsesPerData( 1 ); // get data on every pulse.
  fPrinter2.setNumberRecorded( 1 ); // keep around 20 seconds worth.
  fPrinter2.setFormat( "%s");
  fPrinter2.setDataType( KeyInfoBlock::kString );
  fMonitor.addKeyInfoBlock( fFolder3, fPrinter2 );
}

/**
 * This member function allows adding children to the monitor library. The
 * queriedBlock is the folder that got expanded, so this could be used
 * to add children to.
 */
void MyValueGenerator::updateNodesFor( KeyInfoBlock &queriedBlock,
                                       MonitorLibrary& fMonitor )
{
  static int count = 1;
  if ( queriedBlock == fExpandableFolder )
  {
    // then add a child!
    char displayableName[20];
    char programmaticName[20];

    sprintf( displayableName, "Child %d", count ); 
    sprintf( programmaticName, "CHILD%d", count ); 
    // Note, in this sample we are building the children on the stack
    // so we won't be able to supply unique data for them.
    KeyInfoBlock newChild( displayableName, programmaticName );
    newChild.setBasePulse( 30 * 1000 ); // once every 30 secs
    newChild.setPulsesPerData( 1 ); // get data on every pulse.
    newChild.setNumberRecorded( 1 ); // keep around 30 seconds worth.
    newChild.setFormat( "%s");
    newChild.setDataType( KeyInfoBlock::kString );

    switch ( count )
    {
      case 1:
        fMonitor.addKeyInfoBlock( queriedBlock, newChild );
        break;
      case 2:
        fMonitor.addKeyInfoBlock( queriedBlock, newChild );
        break;
      case 3:
        fMonitor.addKeyInfoBlock( queriedBlock, newChild );
        break;
      case 20:
        fMonitor.addKeyInfoBlock( queriedBlock, newChild );
        break;
      default:
        break;
    }
    count++;
  }
}
